from helper import unittest, PillowTestCase, cached_property

from PIL import Image


class TestImagingPaste(PillowTestCase):
    masks = {}
    size = 128

    def assert_9points_image(self, im, expected):
        expected = [
            point[0]
            if im.mode == 'L' else
            point[:len(im.mode)]
            for point in expected
        ]
        px = im.load()
        actual = [
            px[0, 0],
            px[self.size // 2, 0],
            px[self.size - 1, 0],
            px[0, self.size // 2],
            px[self.size // 2, self.size // 2],
            px[self.size - 1, self.size // 2],
            px[0, self.size - 1],
            px[self.size // 2, self.size - 1],
            px[self.size - 1, self.size - 1],
        ]
        self.assertEqual(actual, expected)

    def assert_9points_paste(self, im, im2, mask, expected):
        im3 = im.copy()
        im3.paste(im2, (0, 0), mask)
        self.assert_9points_image(im3, expected)

        # Abbreviated syntax
        im.paste(im2, mask)
        self.assert_9points_image(im, expected)

    @cached_property
    def mask_1(self):
        mask = Image.new('1', (self.size, self.size))
        px = mask.load()
        for y in range(mask.height):
            for x in range(mask.width):
                px[y, x] = (x + y) % 2
        return mask

    @cached_property
    def mask_L(self):
        return self.gradient_L.transpose(Image.ROTATE_270)

    @cached_property
    def gradient_L(self):
        gradient = Image.new('L', (self.size, self.size))
        px = gradient.load()
        for y in range(gradient.height):
            for x in range(gradient.width):
                px[y, x] = (x + y) % 255
        return gradient

    @cached_property
    def gradient_RGB(self):
        return Image.merge('RGB', [
            self.gradient_L,
            self.gradient_L.transpose(Image.ROTATE_90),
            self.gradient_L.transpose(Image.ROTATE_180),
        ])

    @cached_property
    def gradient_RGBA(self):
        return Image.merge('RGBA', [
            self.gradient_L,
            self.gradient_L.transpose(Image.ROTATE_90),
            self.gradient_L.transpose(Image.ROTATE_180),
            self.gradient_L.transpose(Image.ROTATE_270),
        ])

    @cached_property
    def gradient_RGBa(self):
        return Image.merge('RGBa', [
            self.gradient_L,
            self.gradient_L.transpose(Image.ROTATE_90),
            self.gradient_L.transpose(Image.ROTATE_180),
            self.gradient_L.transpose(Image.ROTATE_270),
        ])

    def test_image_solid(self):
        for mode in ('RGBA', 'RGB', 'L'):
            im = Image.new(mode, (200, 200), 'red')
            im2 = getattr(self, 'gradient_' + mode)

            im.paste(im2, (12, 23))

            im = im.crop((12, 23, im2.width + 12, im2.height + 23))
            self.assert_image_equal(im, im2)

    def test_image_mask_1(self):
        for mode in ('RGBA', 'RGB', 'L'):
            im = Image.new(mode, (200, 200), 'white')
            im2 = getattr(self, 'gradient_' + mode)

            self.assert_9points_paste(im, im2, self.mask_1, [
                (255, 255, 255, 255),
                (255, 255, 255, 255),
                (127, 254, 127, 0),
                (255, 255, 255, 255),
                (255, 255, 255, 255),
                (191, 190, 63, 64),
                (127, 0, 127, 254),
                (191, 64, 63, 190),
                (255, 255, 255, 255),
            ])

    def test_image_mask_L(self):
        for mode in ('RGBA', 'RGB', 'L'):
            im = Image.new(mode, (200, 200), 'white')
            im2 = getattr(self, 'gradient_' + mode)

            self.assert_9points_paste(im, im2, self.mask_L, [
                (128, 191, 255, 191),
                (208, 239, 239, 208),
                (255, 255, 255, 255),
                (112, 111, 206, 207),
                (192, 191, 191, 191),
                (239, 239, 207, 207),
                (128, 1, 128, 254),
                (207, 113, 112, 207),
                (255, 191, 128, 191),
            ])

    def test_image_mask_RGBA(self):
        for mode in ('RGBA', 'RGB', 'L'):
            im = Image.new(mode, (200, 200), 'white')
            im2 = getattr(self, 'gradient_' + mode)

            self.assert_9points_paste(im, im2, self.gradient_RGBA, [
                (128, 191, 255, 191),
                (208, 239, 239, 208),
                (255, 255, 255, 255),
                (112, 111, 206, 207),
                (192, 191, 191, 191),
                (239, 239, 207, 207),
                (128, 1, 128, 254),
                (207, 113, 112, 207),
                (255, 191, 128, 191),
            ])

    def test_image_mask_RGBa(self):
        for mode in ('RGBA', 'RGB', 'L'):
            im = Image.new(mode, (200, 200), 'white')
            im2 = getattr(self, 'gradient_' + mode)

            self.assert_9points_paste(im, im2, self.gradient_RGBa, [
                (128, 255, 126, 255),
                (0, 127, 126, 255),
                (126, 253, 126, 255),
                (128, 127, 254, 255),
                (0, 255, 254, 255),
                (126, 125, 254, 255),
                (128, 1, 128, 255),
                (0, 129, 128, 255),
                (126, 255, 128, 255),
            ])

    def test_color_solid(self):
        for mode in ('RGBA', 'RGB', 'L'):
            im = Image.new(mode, (200, 200), 'black')

            rect = (12, 23, 128 + 12, 128 + 23)
            im.paste('white', rect)

            hist = im.crop(rect).histogram()
            while hist:
                head, hist = hist[:256], hist[256:]
                self.assertEqual(head[255], 128 * 128)
                self.assertEqual(sum(head[:255]), 0)

    def test_color_mask_1(self):
        for mode in ('RGBA', 'RGB', 'L'):
            im = Image.new(mode, (200, 200), (50, 60, 70, 80)[:len(mode)])
            color = (10, 20, 30, 40)[:len(mode)]

            self.assert_9points_paste(im, color, self.mask_1, [
                (50, 60, 70, 80),
                (50, 60, 70, 80),
                (10, 20, 30, 40),
                (50, 60, 70, 80),
                (50, 60, 70, 80),
                (10, 20, 30, 40),
                (10, 20, 30, 40),
                (10, 20, 30, 40),
                (50, 60, 70, 80),
            ])

    def test_color_mask_L(self):
        for mode in ('RGBA', 'RGB', 'L'):
            im = getattr(self, 'gradient_' + mode).copy()
            color = 'white'

            self.assert_9points_paste(im, color, self.mask_L, [
                (127, 191, 254, 191),
                (111, 207, 206, 110),
                (127, 254, 127, 0),
                (207, 207, 239, 239),
                (191, 191, 190, 191),
                (207, 206, 111, 112),
                (254, 254, 254, 255),
                (239, 206, 206, 238),
                (254, 191, 127, 191),
            ])

    def test_color_mask_RGBA(self):
        for mode in ('RGBA', 'RGB', 'L'):
            im = getattr(self, 'gradient_' + mode).copy()
            color = 'white'

            self.assert_9points_paste(im, color, self.gradient_RGBA, [
                (127, 191, 254, 191),
                (111, 207, 206, 110),
                (127, 254, 127, 0),
                (207, 207, 239, 239),
                (191, 191, 190, 191),
                (207, 206, 111, 112),
                (254, 254, 254, 255),
                (239, 206, 206, 238),
                (254, 191, 127, 191),
            ])

    def test_color_mask_RGBa(self):
        for mode in ('RGBA', 'RGB', 'L'):
            im = getattr(self, 'gradient_' + mode).copy()
            color = 'white'

            self.assert_9points_paste(im, color, self.gradient_RGBa, [
                (255, 63, 126, 63),
                (47, 143, 142, 46),
                (126, 253, 126, 255),
                (15, 15, 47, 47),
                (63, 63, 62, 63),
                (142, 141, 46, 47),
                (255, 255, 255, 0),
                (48, 15, 15, 47),
                (126, 63, 255, 63)
            ])

    def test_different_sizes(self):
        im = Image.new('RGB', (100, 100))
        im2 = Image.new('RGB', (50, 50))

        im.copy().paste(im2)
        im.copy().paste(im2, (0, 0))


if __name__ == '__main__':
    unittest.main()



from helper import unittest, PillowTestCase

from PIL import Image

PIL151 = b"""
#define basic_width 32
#define basic_height 32
static char basic_bits[] = {
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00,
0x80, 0xff, 0xff, 0x01, 0x40, 0x00, 0x00, 0x02,
0x20, 0x00, 0x00, 0x04, 0x20, 0x00, 0x00, 0x04, 0x10, 0x00, 0x00, 0x08,
0x10, 0x00, 0x00, 0x08,
0x10, 0x00, 0x00, 0x08, 0x10, 0x00, 0x00, 0x08,
0x10, 0x00, 0x00, 0x08, 0x10, 0x00, 0x00, 0x08, 0x10, 0x00, 0x00, 0x08,
0x20, 0x00, 0x00, 0x04,
0x20, 0x00, 0x00, 0x04, 0x40, 0x00, 0x00, 0x02,
0x80, 0xff, 0xff, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00,
};
"""


class TestFileXbm(PillowTestCase):

    def test_pil151(self):
        from io import BytesIO

        im = Image.open(BytesIO(PIL151))

        im.load()
        self.assertEqual(im.mode, '1')
        self.assertEqual(im.size, (32, 32))

    def test_open(self):
        # Arrange
        # Created with `convert hopper.png hopper.xbm`
        filename = "Tests/images/hopper.xbm"

        # Act
        im = Image.open(filename)

        # Assert
        self.assertEqual(im.mode, '1')
        self.assertEqual(im.size, (128, 128))

    def test_open_filename_with_underscore(self):
        # Arrange
        # Created with `convert hopper.png hopper_underscore.xbm`
        filename = "Tests/images/hopper_underscore.xbm"

        # Act
        im = Image.open(filename)

        # Assert
        self.assertEqual(im.mode, '1')
        self.assertEqual(im.size, (128, 128))


if __name__ == '__main__':
    unittest.main()



"""
Helper functions.
"""
from __future__ import print_function
import sys
import tempfile
import os
import unittest

from PIL import Image, ImageMath

import logging
logger = logging.getLogger(__name__)


HAS_UPLOADER = False

if os.environ.get('SHOW_ERRORS', None):
    # local img.show for errors.
    HAS_UPLOADER = True

    class test_image_results:
        @classmethod
        def upload(self, a, b):
            a.show()
            b.show()
else:
    try:
        import test_image_results
        HAS_UPLOADER = True
    except ImportError:
        pass


def convert_to_comparable(a, b):
    new_a, new_b = a, b
    if a.mode == 'P':
        new_a = Image.new('L', a.size)
        new_b = Image.new('L', b.size)
        new_a.putdata(a.getdata())
        new_b.putdata(b.getdata())
    elif a.mode == 'I;16':
        new_a = a.convert('I')
        new_b = b.convert('I')
    return new_a, new_b


class PillowTestCase(unittest.TestCase):

    def __init__(self, *args, **kwargs):
        unittest.TestCase.__init__(self, *args, **kwargs)
        # holds last result object passed to run method:
        self.currentResult = None

    # Nicer output for --verbose
    def __str__(self):
        return self.__class__.__name__ + "." + self._testMethodName

    def run(self, result=None):
        self.currentResult = result  # remember result for use later
        unittest.TestCase.run(self, result)  # call superclass run method

    def delete_tempfile(self, path):
        try:
            ok = self.currentResult.wasSuccessful()
        except AttributeError:  # for pytest
            ok = True

        if ok:
            # only clean out tempfiles if test passed
            try:
                os.remove(path)
            except OSError:
                pass  # report?
        else:
            print("=== orphaned temp file: %s" % path)

    def assert_deep_equal(self, a, b, msg=None):
        try:
            self.assertEqual(
                len(a), len(b),
                msg or "got length %s, expected %s" % (len(a), len(b)))
            self.assertTrue(
                all(x == y for x, y in zip(a, b)),
                msg or "got %s, expected %s" % (a, b))
        except:
            self.assertEqual(a, b, msg)

    def assert_image(self, im, mode, size, msg=None):
        if mode is not None:
            self.assertEqual(
                im.mode, mode,
                msg or "got mode %r, expected %r" % (im.mode, mode))

        if size is not None:
            self.assertEqual(
                im.size, size,
                msg or "got size %r, expected %r" % (im.size, size))

    def assert_image_equal(self, a, b, msg=None):
        self.assertEqual(
            a.mode, b.mode,
            msg or "got mode %r, expected %r" % (a.mode, b.mode))
        self.assertEqual(
            a.size, b.size,
            msg or "got size %r, expected %r" % (a.size, b.size))
        if a.tobytes() != b.tobytes():
            if HAS_UPLOADER:
                try:
                    url = test_image_results.upload(a, b)
                    logger.error("Url for test images: %s" % url)
                except Exception as msg:
                    pass

            self.fail(msg or "got different content")

    def assert_image_equal_tofile(self, a, filename, msg=None, mode=None):
        with Image.open(filename) as img:
            if mode:
                img = img.convert(mode)
            self.assert_image_equal(a, img, msg)

    def assert_image_similar(self, a, b, epsilon, msg=None):
        epsilon = float(epsilon)
        self.assertEqual(
            a.mode, b.mode,
            msg or "got mode %r, expected %r" % (a.mode, b.mode))
        self.assertEqual(
            a.size, b.size,
            msg or "got size %r, expected %r" % (a.size, b.size))

        a, b = convert_to_comparable(a, b)

        diff = 0
        for ach, bch in zip(a.split(), b.split()):
            chdiff = ImageMath.eval("abs(a - b)", a=ach, b=bch).convert('L')
            diff += sum(i * num for i, num in enumerate(chdiff.histogram()))

        ave_diff = float(diff)/(a.size[0]*a.size[1])
        try:
            self.assertGreaterEqual(
                epsilon, ave_diff,
                (msg or '') +
                " average pixel value difference %.4f > epsilon %.4f" % (
                    ave_diff, epsilon))
        except Exception as e:
            if HAS_UPLOADER:
                try:
                    url = test_image_results.upload(a, b)
                    logger.error("Url for test images: %s" % url)
                except:
                    pass
            raise e

    def assert_image_similar_tofile(self, a, filename, epsilon, msg=None, mode=None):
        with Image.open(filename) as img:
            if mode:
                img = img.convert(mode)
            self.assert_image_similar(a, img, epsilon, msg)

    def assert_warning(self, warn_class, func, *args, **kwargs):
        import warnings

        result = None
        with warnings.catch_warnings(record=True) as w:
            # Cause all warnings to always be triggered.
            warnings.simplefilter("always")

            # Hopefully trigger a warning.
            result = func(*args, **kwargs)

            # Verify some things.
            if warn_class is None:
                self.assertEqual(len(w), 0,
                                 "Expected no warnings, got %s" %
                                 list(v.category for v in w))
            else:
                self.assertGreaterEqual(len(w), 1)
                found = False
                for v in w:
                    if issubclass(v.category, warn_class):
                        found = True
                        break
                self.assertTrue(found)
        return result

    def assert_all_same(self, items, msg=None):
        self.assertTrue(items.count(items[0]) == len(items), msg)

    def assert_not_all_same(self, items, msg=None):
        self.assertFalse(items.count(items[0]) == len(items), msg)

    def skipKnownBadTest(self, msg=None, platform=None,
                         travis=None, interpreter=None):
        # Skip if platform/travis matches, and
        # PILLOW_RUN_KNOWN_BAD is not true in the environment.
        if os.environ.get('PILLOW_RUN_KNOWN_BAD', False):
            print(os.environ.get('PILLOW_RUN_KNOWN_BAD', False))
            return

        skip = True
        if platform is not None:
            skip = sys.platform.startswith(platform)
        if travis is not None:
            skip = skip and (travis == bool(os.environ.get('TRAVIS', False)))
        if interpreter is not None:
            skip = skip and (interpreter == 'pypy' and
                             hasattr(sys, 'pypy_version_info'))
        if skip:
            self.skipTest(msg or "Known Bad Test")

    def tempfile(self, template):
        assert template[:5] in ("temp.", "temp_")
        fd, path = tempfile.mkstemp(template[4:], template[:4])
        os.close(fd)

        self.addCleanup(self.delete_tempfile, path)
        return path

    def open_withImagemagick(self, f):
        if not imagemagick_available():
            raise IOError()

        outfile = self.tempfile("temp.png")
        if command_succeeds([IMCONVERT, f, outfile]):
            return Image.open(outfile)
        raise IOError()


@unittest.skipIf(sys.platform.startswith('win32'), "requires Unix or macOS")
class PillowLeakTestCase(PillowTestCase):
    # requires unix/osx
    iterations = 100  # count
    mem_limit = 512  # k

    def _get_mem_usage(self):
        """
        Gets the RUSAGE memory usage, returns in K. Encapsulates the difference
        between OSX and Linux rss reporting

        :returns; memory usage in kilobytes
        """

        from resource import getrusage, RUSAGE_SELF
        mem = getrusage(RUSAGE_SELF).ru_maxrss
        if sys.platform == 'darwin':
            # man 2 getrusage:
            #     ru_maxrss    the maximum resident set size utilized (in bytes).
            return mem / 1024  # Kb
        else:
            # linux
            # man 2 getrusage
            #        ru_maxrss (since Linux 2.6.32)
            #  This is the maximum resident set size used (in kilobytes).
            return mem  # Kb

    def _test_leak(self, core):
        start_mem = self._get_mem_usage()
        for cycle in range(self.iterations):
            core()
            mem = (self._get_mem_usage() - start_mem)
            self.assertLess(mem, self.mem_limit,
                            msg='memory usage limit exceeded in iteration %d' % cycle)


# helpers

py3 = sys.version_info.major >= 3


def fromstring(data):
    from io import BytesIO
    return Image.open(BytesIO(data))


def tostring(im, string_format, **options):
    from io import BytesIO
    out = BytesIO()
    im.save(out, string_format, **options)
    return out.getvalue()


def hopper(mode=None, cache={}):
    if mode is None:
        # Always return fresh not-yet-loaded version of image.
        # Operations on not-yet-loaded images is separate class of errors
        # what we should catch.
        return Image.open("Tests/images/hopper.ppm")
    # Use caching to reduce reading from disk but so an original copy is
    # returned each time and the cached image isn't modified by tests
    # (for fast, isolated, repeatable tests).
    im = cache.get(mode)
    if im is None:
        if mode == "F":
            im = hopper("L").convert(mode)
        elif mode[:4] == "I;16":
            im = hopper("I").convert(mode)
        else:
            im = hopper().convert(mode)
        cache[mode] = im
    return im.copy()


def command_succeeds(cmd):
    """
    Runs the command, which must be a list of strings. Returns True if the
    command succeeds, or False if an OSError was raised by subprocess.Popen.
    """
    import subprocess
    with open(os.devnull, 'wb') as f:
        try:
            subprocess.call(cmd, stdout=f, stderr=subprocess.STDOUT)
        except OSError:
            return False
    return True


def djpeg_available():
    return command_succeeds(['djpeg', '-version'])


def cjpeg_available():
    return command_succeeds(['cjpeg', '-version'])


def netpbm_available():
    return (command_succeeds(["ppmquant", "--version"]) and
            command_succeeds(["ppmtogif", "--version"]))


def imagemagick_available():
    return IMCONVERT and command_succeeds([IMCONVERT, '-version'])


def on_appveyor():
    return 'APPVEYOR' in os.environ


if sys.platform == 'win32':
    IMCONVERT = os.environ.get('MAGICK_HOME', '')
    if IMCONVERT:
        IMCONVERT = os.path.join(IMCONVERT, 'convert.exe')
else:
    IMCONVERT = 'convert'


def distro():
    if os.path.exists('/etc/os-release'):
        with open('/etc/os-release', 'r') as f:
            for line in f:
                if 'ID=' in line:
                    return line.strip().split('=')[1]


class cached_property(object):
    def __init__(self, func):
        self.func = func

    def __get__(self, instance, cls=None):
        result = instance.__dict__[self.func.__name__] = self.func(instance)
        return result



from helper import unittest, PillowTestCase
from PIL import Image

TEST_FILE = "Tests/images/libtiff_segfault.tif"


class TestLibtiffSegfault(PillowTestCase):
    def test_segfault(self):
        """ This test should not segfault. It will on Pillow <= 3.1.0 and
            libtiff >= 4.0.0
            """

        with self.assertRaises(IOError):
            im = Image.open(TEST_FILE)
            im.load()


if __name__ == '__main__':
    unittest.main()



from helper import unittest, PillowTestCase

from PIL import Image, TarIO

codecs = dir(Image.core)

# Sample tar archive
TEST_TAR_FILE = "Tests/images/hopper.tar"


class TestFileTar(PillowTestCase):

    def setUp(self):
        if "zip_decoder" not in codecs and "jpeg_decoder" not in codecs:
            self.skipTest("neither jpeg nor zip support not available")

    def test_sanity(self):
        if "zip_decoder" in codecs:
            tar = TarIO.TarIO(TEST_TAR_FILE, 'hopper.png')
            im = Image.open(tar)
            im.load()
            self.assertEqual(im.mode, "RGB")
            self.assertEqual(im.size, (128, 128))
            self.assertEqual(im.format, "PNG")

        if "jpeg_decoder" in codecs:
            tar = TarIO.TarIO(TEST_TAR_FILE, 'hopper.jpg')
            im = Image.open(tar)
            im.load()
            self.assertEqual(im.mode, "RGB")
            self.assertEqual(im.size, (128, 128))
            self.assertEqual(im.format, "JPEG")


if __name__ == '__main__':
    unittest.main()



from helper import PillowTestCase, unittest
import sys

from PIL import Image


@unittest.skipIf(sys.platform.startswith('win32'), "Win32 does not call map_buffer")
class TestMap(PillowTestCase):
    def test_overflow(self):
        # There is the potential to overflow comparisons in map.c
        # if there are > SIZE_MAX bytes in the image or if
        # the file encodes an offset that makes
        # (offset + size(bytes)) > SIZE_MAX

        # Note that this image triggers the decompression bomb warning:
        max_pixels = Image.MAX_IMAGE_PIXELS
        Image.MAX_IMAGE_PIXELS = None

        # This image hits the offset test.
        im = Image.open('Tests/images/l2rgb_read.bmp')
        with self.assertRaises((ValueError, MemoryError, IOError)):
            im.load()

        Image.MAX_IMAGE_PIXELS = max_pixels


if __name__ == '__main__':
    unittest.main()



from helper import unittest, PillowTestCase

from PIL import Image, GbrImagePlugin


class TestFileGbr(PillowTestCase):

    def test_invalid_file(self):
        invalid_file = "Tests/images/flower.jpg"

        self.assertRaises(SyntaxError,
                          GbrImagePlugin.GbrImageFile, invalid_file)

    def test_gbr_file(self):
        im = Image.open('Tests/images/gbr.gbr')

        target = Image.open('Tests/images/gbr.png')

        self.assert_image_equal(target, im)


if __name__ == '__main__':
    unittest.main()



from helper import unittest, PillowTestCase, hopper

from PIL import Image

TEST_FILE = "Tests/images/hopper.ppm"

ORIGINAL_LIMIT = Image.MAX_IMAGE_PIXELS


class TestDecompressionBomb(PillowTestCase):

    def tearDown(self):
        Image.MAX_IMAGE_PIXELS = ORIGINAL_LIMIT

    def test_no_warning_small_file(self):
        # Implicit assert: no warning.
        # A warning would cause a failure.
        Image.open(TEST_FILE)

    def test_no_warning_no_limit(self):
        # Arrange
        # Turn limit off
        Image.MAX_IMAGE_PIXELS = None
        self.assertIsNone(Image.MAX_IMAGE_PIXELS)

        # Act / Assert
        # Implicit assert: no warning.
        # A warning would cause a failure.
        Image.open(TEST_FILE)

    def test_warning(self):
        # Set limit to trigger warning on the test file
        Image.MAX_IMAGE_PIXELS = 128 * 128 - 1
        self.assertEqual(Image.MAX_IMAGE_PIXELS, 128 * 128 - 1)

        self.assert_warning(Image.DecompressionBombWarning,
                            Image.open, TEST_FILE)

    def test_exception(self):
        # Set limit to trigger exception on the test file
        Image.MAX_IMAGE_PIXELS = 64 * 128 - 1
        self.assertEqual(Image.MAX_IMAGE_PIXELS, 64 * 128 - 1)

        self.assertRaises(Image.DecompressionBombError,
                          lambda: Image.open(TEST_FILE))


class TestDecompressionCrop(PillowTestCase):

    def setUp(self):
        self.src = hopper()
        Image.MAX_IMAGE_PIXELS = self.src.height * self.src.width * 4 - 1

    def tearDown(self):
        Image.MAX_IMAGE_PIXELS = ORIGINAL_LIMIT

    def testEnlargeCrop(self):
        # Crops can extend the extents, therefore we should have the
        # same decompression bomb warnings on them.
        box = (0, 0, self.src.width * 2, self.src.height * 2)
        self.assert_warning(Image.DecompressionBombWarning,
                            self.src.crop, box)


if __name__ == '__main__':
    unittest.main()



from helper import unittest, PillowTestCase, hopper

from PIL import Image

im = hopper().resize((128, 100))


class TestImageArray(PillowTestCase):

    def test_toarray(self):
        def test(mode):
            ai = im.convert(mode).__array_interface__
            return ai['version'], ai["shape"], ai["typestr"], len(ai["data"])
        # self.assertEqual(test("1"), (3, (100, 128), '|b1', 1600))
        self.assertEqual(test("L"), (3, (100, 128), '|u1', 12800))

        # FIXME: wrong?
        self.assertEqual(test("I"), (3, (100, 128), Image._ENDIAN + 'i4', 51200))
        # FIXME: wrong?
        self.assertEqual(test("F"), (3, (100, 128), Image._ENDIAN + 'f4', 51200))

        self.assertEqual(test("LA"), (3, (100, 128, 2), '|u1', 25600))
        self.assertEqual(test("RGB"), (3, (100, 128, 3), '|u1', 38400))
        self.assertEqual(test("RGBA"), (3, (100, 128, 4), '|u1', 51200))
        self.assertEqual(test("RGBX"), (3, (100, 128, 4), '|u1', 51200))

    def test_fromarray(self):

        class Wrapper(object):
            """ Class with API matching Image.fromarray """

            def __init__(self, img, arr_params):
                self.img = img
                self.__array_interface__ = arr_params

            def tobytes(self):
                return self.img.tobytes()

        def test(mode):
            i = im.convert(mode)
            a = i.__array_interface__
            a["strides"] = 1  # pretend it's non-contiguous
            # Make wrapper instance for image, new array interface
            wrapped = Wrapper(i, a)
            out = Image.fromarray(wrapped)
            return out.mode, out.size, list(i.getdata()) == list(out.getdata())

        # self.assertEqual(test("1"), ("1", (128, 100), True))
        self.assertEqual(test("L"), ("L", (128, 100), True))
        self.assertEqual(test("I"), ("I", (128, 100), True))
        self.assertEqual(test("F"), ("F", (128, 100), True))
        self.assertEqual(test("LA"), ("LA", (128, 100), True))
        self.assertEqual(test("RGB"), ("RGB", (128, 100), True))
        self.assertEqual(test("RGBA"), ("RGBA", (128, 100), True))
        self.assertEqual(test("RGBX"), ("RGBA", (128, 100), True))


if __name__ == '__main__':
    unittest.main()



from helper import unittest, PillowTestCase, hopper

from PIL import Image


class TestImageFromBytes(PillowTestCase):

    def test_sanity(self):
        im1 = hopper()
        im2 = Image.frombytes(im1.mode, im1.size, im1.tobytes())

        self.assert_image_equal(im1, im2)

    def test_not_implemented(self):
        self.assertRaises(NotImplementedError, Image.fromstring)


if __name__ == '__main__':
    unittest.main()



from helper import unittest, PillowTestCase, hopper

from PIL import Image, SunImagePlugin

import os

EXTRA_DIR = 'Tests/images/sunraster'


class TestFileSun(PillowTestCase):

    def test_sanity(self):
        # Arrange
        # Created with ImageMagick: convert hopper.jpg hopper.ras
        test_file = "Tests/images/hopper.ras"

        # Act
        im = Image.open(test_file)

        # Assert
        self.assertEqual(im.size, (128, 128))

        self.assert_image_similar(im, hopper(), 5)  # visually verified

        invalid_file = "Tests/images/flower.jpg"
        self.assertRaises(SyntaxError,
                          SunImagePlugin.SunImageFile, invalid_file)

    def test_im1(self):
        im = Image.open('Tests/images/sunraster.im1')
        target = Image.open('Tests/images/sunraster.im1.png')
        self.assert_image_equal(im, target)

    @unittest.skipIf(not os.path.exists(EXTRA_DIR),
                     "Extra image files not installed")
    def test_others(self):
        files = (os.path.join(EXTRA_DIR, f) for f in
                 os.listdir(EXTRA_DIR) if os.path.splitext(f)[1]
                 in ('.sun', '.SUN', '.ras'))
        for path in files:
            with Image.open(path) as im:
                im.load()
                self.assertIsInstance(im, SunImagePlugin.SunImageFile)
                target_path = "%s.png" % os.path.splitext(path)[0]
                # im.save(target_file)
                with Image.open(target_path) as target:
                    self.assert_image_equal(im, target)


if __name__ == '__main__':
    unittest.main()



#
# The Python Imaging Library.
# $Id$
#
# EXIF tags
#
# Copyright (c) 2003 by Secret Labs AB
#
# See the README file for information on usage and redistribution.
#

##
# This module provides constants and clear-text names for various
# well-known EXIF tags.
##

##
# Maps EXIF tags to tag names.

TAGS = {

    # possibly incomplete
    0x000b: "ProcessingSoftware",
    0x00fe: "NewSubfileType",
    0x00ff: "SubfileType",
    0x0100: "ImageWidth",
    0x0101: "ImageLength",
    0x0102: "BitsPerSample",
    0x0103: "Compression",
    0x0106: "PhotometricInterpretation",
    0x0107: "Thresholding",
    0x0108: "CellWidth",
    0x0109: "CellLength",
    0x010a: "FillOrder",
    0x010d: "DocumentName",
    0x010e: "ImageDescription",
    0x010f: "Make",
    0x0110: "Model",
    0x0111: "StripOffsets",
    0x0112: "Orientation",
    0x0115: "SamplesPerPixel",
    0x0116: "RowsPerStrip",
    0x0117: "StripByteCounts",
    0x0118: "MinSampleValue",
    0x0119: "MaxSampleValue",
    0x011a: "XResolution",
    0x011b: "YResolution",
    0x011c: "PlanarConfiguration",
    0x011d: "PageName",
    0x0120: "FreeOffsets",
    0x0121: "FreeByteCounts",
    0x0122: "GrayResponseUnit",
    0x0123: "GrayResponseCurve",
    0x0124: "T4Options",
    0x0125: "T6Options",
    0x0128: "ResolutionUnit",
    0x0129: "PageNumber",
    0x012d: "TransferFunction",
    0x0131: "Software",
    0x0132: "DateTime",
    0x013b: "Artist",
    0x013c: "HostComputer",
    0x013d: "Predictor",
    0x013e: "WhitePoint",
    0x013f: "PrimaryChromaticities",
    0x0140: "ColorMap",
    0x0141: "HalftoneHints",
    0x0142: "TileWidth",
    0x0143: "TileLength",
    0x0144: "TileOffsets",
    0x0145: "TileByteCounts",
    0x014a: "SubIFDs",
    0x014c: "InkSet",
    0x014d: "InkNames",
    0x014e: "NumberOfInks",
    0x0150: "DotRange",
    0x0151: "TargetPrinter",
    0x0152: "ExtraSamples",
    0x0153: "SampleFormat",
    0x0154: "SMinSampleValue",
    0x0155: "SMaxSampleValue",
    0x0156: "TransferRange",
    0x0157: "ClipPath",
    0x0158: "XClipPathUnits",
    0x0159: "YClipPathUnits",
    0x015a: "Indexed",
    0x015b: "JPEGTables",
    0x015f: "OPIProxy",
    0x0200: "JPEGProc",
    0x0201: "JpegIFOffset",
    0x0202: "JpegIFByteCount",
    0x0203: "JpegRestartInterval",
    0x0205: "JpegLosslessPredictors",
    0x0206: "JpegPointTransforms",
    0x0207: "JpegQTables",
    0x0208: "JpegDCTables",
    0x0209: "JpegACTables",
    0x0211: "YCbCrCoefficients",
    0x0212: "YCbCrSubSampling",
    0x0213: "YCbCrPositioning",
    0x0214: "ReferenceBlackWhite",
    0x02bc: "XMLPacket",
    0x1000: "RelatedImageFileFormat",
    0x1001: "RelatedImageWidth",
    0x1002: "RelatedImageLength",
    0x4746: "Rating",
    0x4749: "RatingPercent",
    0x800d: "ImageID",
    0x828d: "CFARepeatPatternDim",
    0x828e: "CFAPattern",
    0x828f: "BatteryLevel",
    0x8298: "Copyright",
    0x829a: "ExposureTime",
    0x829d: "FNumber",
    0x83bb: "IPTCNAA",
    0x8649: "ImageResources",
    0x8769: "ExifOffset",
    0x8773: "InterColorProfile",
    0x8822: "ExposureProgram",
    0x8824: "SpectralSensitivity",
    0x8825: "GPSInfo",
    0x8827: "ISOSpeedRatings",
    0x8828: "OECF",
    0x8829: "Interlace",
    0x882a: "TimeZoneOffset",
    0x882b: "SelfTimerMode",
    0x9000: "ExifVersion",
    0x9003: "DateTimeOriginal",
    0x9004: "DateTimeDigitized",
    0x9101: "ComponentsConfiguration",
    0x9102: "CompressedBitsPerPixel",
    0x9201: "ShutterSpeedValue",
    0x9202: "ApertureValue",
    0x9203: "BrightnessValue",
    0x9204: "ExposureBiasValue",
    0x9205: "MaxApertureValue",
    0x9206: "SubjectDistance",
    0x9207: "MeteringMode",
    0x9208: "LightSource",
    0x9209: "Flash",
    0x920a: "FocalLength",
    0x920b: "FlashEnergy",
    0x920c: "SpatialFrequencyResponse",
    0x920d: "Noise",
    0x9211: "ImageNumber",
    0x9212: "SecurityClassification",
    0x9213: "ImageHistory",
    0x9214: "SubjectLocation",
    0x9215: "ExposureIndex",
    0x9216: "TIFF/EPStandardID",
    0x927c: "MakerNote",
    0x9286: "UserComment",
    0x9290: "SubsecTime",
    0x9291: "SubsecTimeOriginal",
    0x9292: "SubsecTimeDigitized",
    0x9c9b: "XPTitle",
    0x9c9c: "XPComment",
    0x9c9d: "XPAuthor",
    0x9c9e: "XPKeywords",
    0x9c9f: "XPSubject",
    0xa000: "FlashPixVersion",
    0xa001: "ColorSpace",
    0xa002: "ExifImageWidth",
    0xa003: "ExifImageHeight",
    0xa004: "RelatedSoundFile",
    0xa005: "ExifInteroperabilityOffset",
    0xa20b: "FlashEnergy",
    0xa20c: "SpatialFrequencyResponse",
    0xa20e: "FocalPlaneXResolution",
    0xa20f: "FocalPlaneYResolution",
    0xa210: "FocalPlaneResolutionUnit",
    0xa214: "SubjectLocation",
    0xa215: "ExposureIndex",
    0xa217: "SensingMethod",
    0xa300: "FileSource",
    0xa301: "SceneType",
    0xa302: "CFAPattern",
    0xa401: "CustomRendered",
    0xa402: "ExposureMode",
    0xa403: "WhiteBalance",
    0xa404: "DigitalZoomRatio",
    0xa405: "FocalLengthIn35mmFilm",
    0xa406: "SceneCaptureType",
    0xa407: "GainControl",
    0xa408: "Contrast",
    0xa409: "Saturation",
    0xa40a: "Sharpness",
    0xa40b: "DeviceSettingDescription",
    0xa40c: "SubjectDistanceRange",
    0xa420: "ImageUniqueID",
    0xa430: "CameraOwnerName",
    0xa431: "BodySerialNumber",
    0xa432: "LensSpecification",
    0xa433: "LensMake",
    0xa434: "LensModel",
    0xa435: "LensSerialNumber",
    0xa500: "Gamma",
    0xc4a5: "PrintImageMatching",
    0xc612: "DNGVersion",
    0xc613: "DNGBackwardVersion",
    0xc614: "UniqueCameraModel",
    0xc615: "LocalizedCameraModel",
    0xc616: "CFAPlaneColor",
    0xc617: "CFALayout",
    0xc618: "LinearizationTable",
    0xc619: "BlackLevelRepeatDim",
    0xc61a: "BlackLevel",
    0xc61b: "BlackLevelDeltaH",
    0xc61c: "BlackLevelDeltaV",
    0xc61d: "WhiteLevel",
    0xc61e: "DefaultScale",
    0xc61f: "DefaultCropOrigin",
    0xc620: "DefaultCropSize",
    0xc621: "ColorMatrix1",
    0xc622: "ColorMatrix2",
    0xc623: "CameraCalibration1",
    0xc624: "CameraCalibration2",
    0xc625: "ReductionMatrix1",
    0xc626: "ReductionMatrix2",
    0xc627: "AnalogBalance",
    0xc628: "AsShotNeutral",
    0xc629: "AsShotWhiteXY",
    0xc62a: "BaselineExposure",
    0xc62b: "BaselineNoise",
    0xc62c: "BaselineSharpness",
    0xc62d: "BayerGreenSplit",
    0xc62e: "LinearResponseLimit",
    0xc62f: "CameraSerialNumber",
    0xc630: "LensInfo",
    0xc631: "ChromaBlurRadius",
    0xc632: "AntiAliasStrength",
    0xc633: "ShadowScale",
    0xc634: "DNGPrivateData",
    0xc635: "MakerNoteSafety",
    0xc65a: "CalibrationIlluminant1",
    0xc65b: "CalibrationIlluminant2",
    0xc65c: "BestQualityScale",
    0xc65d: "RawDataUniqueID",
    0xc68b: "OriginalRawFileName",
    0xc68c: "OriginalRawFileData",
    0xc68d: "ActiveArea",
    0xc68e: "MaskedAreas",
    0xc68f: "AsShotICCProfile",
    0xc690: "AsShotPreProfileMatrix",
    0xc691: "CurrentICCProfile",
    0xc692: "CurrentPreProfileMatrix",
    0xc6bf: "ColorimetricReference",
    0xc6f3: "CameraCalibrationSignature",
    0xc6f4: "ProfileCalibrationSignature",
    0xc6f6: "AsShotProfileName",
    0xc6f7: "NoiseReductionApplied",
    0xc6f8: "ProfileName",
    0xc6f9: "ProfileHueSatMapDims",
    0xc6fa: "ProfileHueSatMapData1",
    0xc6fb: "ProfileHueSatMapData2",
    0xc6fc: "ProfileToneCurve",
    0xc6fd: "ProfileEmbedPolicy",
    0xc6fe: "ProfileCopyright",
    0xc714: "ForwardMatrix1",
    0xc715: "ForwardMatrix2",
    0xc716: "PreviewApplicationName",
    0xc717: "PreviewApplicationVersion",
    0xc718: "PreviewSettingsName",
    0xc719: "PreviewSettingsDigest",
    0xc71a: "PreviewColorSpace",
    0xc71b: "PreviewDateTime",
    0xc71c: "RawImageDigest",
    0xc71d: "OriginalRawFileDigest",
    0xc71e: "SubTileBlockSize",
    0xc71f: "RowInterleaveFactor",
    0xc725: "ProfileLookTableDims",
    0xc726: "ProfileLookTableData",
    0xc740: "OpcodeList1",
    0xc741: "OpcodeList2",
    0xc74e: "OpcodeList3",
    0xc761: "NoiseProfile"
}

##
# Maps EXIF GPS tags to tag names.

GPSTAGS = {
    0: "GPSVersionID",
    1: "GPSLatitudeRef",
    2: "GPSLatitude",
    3: "GPSLongitudeRef",
    4: "GPSLongitude",
    5: "GPSAltitudeRef",
    6: "GPSAltitude",
    7: "GPSTimeStamp",
    8: "GPSSatellites",
    9: "GPSStatus",
    10: "GPSMeasureMode",
    11: "GPSDOP",
    12: "GPSSpeedRef",
    13: "GPSSpeed",
    14: "GPSTrackRef",
    15: "GPSTrack",
    16: "GPSImgDirectionRef",
    17: "GPSImgDirection",
    18: "GPSMapDatum",
    19: "GPSDestLatitudeRef",
    20: "GPSDestLatitude",
    21: "GPSDestLongitudeRef",
    22: "GPSDestLongitude",
    23: "GPSDestBearingRef",
    24: "GPSDestBearing",
    25: "GPSDestDistanceRef",
    26: "GPSDestDistance",
    27: "GPSProcessingMethod",
    28: "GPSAreaInformation",
    29: "GPSDateStamp",
    30: "GPSDifferential",
    31: "GPSHPositioningError",
}



#!/usr/bin/env python3

import subprocess
import shutil
import sys
import getopt
import os

from config import (compilers, compiler_from_env, pythons, pyversion_from_env,
                    VIRT_BASE, X64_EXT)


def setup_vms():
    ret = []
    for py in pythons:
        for arch in ('', X64_EXT):
            ret.append("virtualenv -p c:/Python%s%s/python.exe --clear %s%s%s"
                       % (py, arch, VIRT_BASE, py, arch))
            ret.append(r"%s%s%s\Scripts\pip.exe install pytest pytest-cov" %
                       (VIRT_BASE, py, arch))
    return "\n".join(ret)


def run_script(params):
    (version, script) = params
    try:
        print("Running %s" % version)
        filename = 'build_pillow_%s.cmd' % version
        with open(filename, 'w') as f:
            f.write(script)

        command = ['powershell', "./%s" % filename]
        proc = subprocess.Popen(command,
                                stdin=subprocess.PIPE,
                                stdout=subprocess.PIPE,
                                stderr=subprocess.PIPE,
                                )
        (trace, stderr) = proc.communicate()
        status = proc.returncode
        print("-- stderr --")
        print(stderr)
        print("-- stdout --")
        print(trace)
        print("Done with %s: %s" % (version, status))
        return (version, status, trace, stderr)
    except Exception as msg:
        print("Error with %s: %s" % (version, str(msg)))
        return (version, -1, "", str(msg))


def header(op):
    return r"""
setlocal
set MPLSRC=%%~dp0\..
set INCLIB=%%~dp0\depends
set BLDOPT=%s
cd /D %%MPLSRC%%
""" % (op)


def footer():
    return """endlocal
exit
"""


def build_one(py_ver, compiler):
    # UNDONE virtual envs if we're not running on appveyor
    args = {}
    args.update(compiler)
    if 'PYTHON' in os.environ:
        args['python_path'] = "%PYTHON%"
    else:
        args['python_path'] = "%s%s\\Scripts" % (VIRT_BASE, py_ver)

    args['executable'] = "python.exe"
    if 'EXECUTABLE' in os.environ:
        args['executable'] = "%EXECUTABLE%"

    args['py_ver'] = py_ver
    if '34' in py_ver:
        args['tcl_ver'] = '86'
    else:
        args['tcl_ver'] = '85'

    return r"""
setlocal EnableDelayedExpansion
call "%%ProgramFiles%%\Microsoft SDKs\Windows\%(env_version)s\Bin\SetEnv.Cmd" /Release %(env_flags)s
set DISTUTILS_USE_SDK=1
set LIB=%%LIB%%;%%INCLIB%%\%(inc_dir)s
set INCLUDE=%%INCLUDE%%;%%INCLIB%%\%(inc_dir)s;%%INCLIB%%\tcl%(tcl_ver)s\include

setlocal
set LIB=%%LIB%%;C:\Python%(py_ver)s\tcl
call %(python_path)s\%(executable)s setup.py %%BLDOPT%%
endlocal

endlocal
""" % args


def clean():
    try:
        shutil.rmtree('../build')
    except:
        # could already be removed
        pass
    run_script(('virtualenvs', setup_vms()))


def main(op):
    scripts = []

    for py_version, compiler_version in pythons.items():
        scripts.append((py_version,
                        "\n".join([header(op),
                                   build_one(py_version,
                                             compilers[(compiler_version,
                                                        32)]),
                                   footer()])))

        scripts.append(("%s%s" % (py_version, X64_EXT),
                        "\n".join([header(op),
                                   build_one("%sx64" % py_version,
                                             compilers[(compiler_version,
                                                        64)]),
                                   footer()])))

    results = map(run_script, scripts)

    for (version, status, trace, err) in results:
        print("Compiled %s: %s" % (version, status and 'ERR' or 'OK'))


def run_one(op):

    compiler = compiler_from_env()
    py_version = pyversion_from_env()

    run_script((py_version,
                "\n".join([header(op),
                           build_one(py_version, compiler),
                           footer()])
                ))


if __name__ == '__main__':
    opts, args = getopt.getopt(sys.argv[1:], '', ['clean', 'dist', 'wheel'])
    opts = dict(opts)

    if '--clean' in opts:
        clean()

    op = 'install'
    if '--dist' in opts:
        op = "bdist_wininst --user-access-control=auto"
    elif '--wheel' in opts:
        op = "bdist_wheel"

    if 'PYTHON' in os.environ:
        run_one(op)
    else:
        main(op)



from helper import unittest, PillowTestCase, hopper
from test_imageqt import PillowQtTestCase, PillowQPixmapTestCase

from PIL import ImageQt


class TestFromQPixmap(PillowQPixmapTestCase, PillowTestCase):

    def roundtrip(self, expected):
        PillowQtTestCase.setUp(self)
        result = ImageQt.fromqpixmap(ImageQt.toqpixmap(expected))
        # Qt saves all pixmaps as rgb
        self.assert_image_equal(result, expected.convert('RGB'))

    def test_sanity_1(self):
        self.roundtrip(hopper('1'))

    def test_sanity_rgb(self):
        self.roundtrip(hopper('RGB'))

    def test_sanity_rgba(self):
        self.roundtrip(hopper('RGBA'))

    def test_sanity_l(self):
        self.roundtrip(hopper('L'))

    def test_sanity_p(self):
        self.roundtrip(hopper('P'))


if __name__ == '__main__':
    unittest.main()



from helper import unittest, PillowTestCase
from io import BytesIO
from PIL import Image


test_files = ["Tests/images/sugarshack.mpo", "Tests/images/frozenpond.mpo"]


class TestFileMpo(PillowTestCase):

    def setUp(self):
        codecs = dir(Image.core)
        if "jpeg_encoder" not in codecs or "jpeg_decoder" not in codecs:
            self.skipTest("jpeg support not available")

    def frame_roundtrip(self, im, **options):
        # Note that for now, there is no MPO saving functionality
        out = BytesIO()
        im.save(out, "MPO", **options)
        test_bytes = out.tell()
        out.seek(0)
        im = Image.open(out)
        im.bytes = test_bytes  # for testing only
        return im

    def test_sanity(self):
        for test_file in test_files:
            im = Image.open(test_file)
            im.load()
            self.assertEqual(im.mode, "RGB")
            self.assertEqual(im.size, (640, 480))
            self.assertEqual(im.format, "MPO")

    def test_app(self):
        for test_file in test_files:
            # Test APP/COM reader (@PIL135)
            im = Image.open(test_file)
            self.assertEqual(im.applist[0][0], 'APP1')
            self.assertEqual(im.applist[1][0], 'APP2')
            self.assertEqual(im.applist[1][1][:16],
                             b'MPF\x00MM\x00*\x00\x00\x00\x08\x00\x03\xb0\x00')
            self.assertEqual(len(im.applist), 2)

    def test_exif(self):
        for test_file in test_files:
            im = Image.open(test_file)
            info = im._getexif()
            self.assertEqual(info[272], 'Nintendo 3DS')
            self.assertEqual(info[296], 2)
            self.assertEqual(info[34665], 188)

    def test_mp(self):
        for test_file in test_files:
            im = Image.open(test_file)
            mpinfo = im._getmp()
            self.assertEqual(mpinfo[45056], b'0100')
            self.assertEqual(mpinfo[45057], 2)

    def test_mp_attribute(self):
        for test_file in test_files:
            im = Image.open(test_file)
            mpinfo = im._getmp()
            frameNumber = 0
            for mpentry in mpinfo[45058]:
                mpattr = mpentry['Attribute']
                if frameNumber:
                    self.assertFalse(mpattr['RepresentativeImageFlag'])
                else:
                    self.assertTrue(mpattr['RepresentativeImageFlag'])
                self.assertFalse(mpattr['DependentParentImageFlag'])
                self.assertFalse(mpattr['DependentChildImageFlag'])
                self.assertEqual(mpattr['ImageDataFormat'], 'JPEG')
                self.assertEqual(mpattr['MPType'],
                                 'Multi-Frame Image: (Disparity)')
                self.assertEqual(mpattr['Reserved'], 0)
                frameNumber += 1

    def test_seek(self):
        for test_file in test_files:
            im = Image.open(test_file)
            self.assertEqual(im.tell(), 0)
            # prior to first image raises an error, both blatant and borderline
            self.assertRaises(EOFError, im.seek, -1)
            self.assertRaises(EOFError, im.seek, -523)
            # after the final image raises an error,
            # both blatant and borderline
            self.assertRaises(EOFError, im.seek, 2)
            self.assertRaises(EOFError, im.seek, 523)
            # bad calls shouldn't change the frame
            self.assertEqual(im.tell(), 0)
            # this one will work
            im.seek(1)
            self.assertEqual(im.tell(), 1)
            # and this one, too
            im.seek(0)
            self.assertEqual(im.tell(), 0)

    def test_n_frames(self):
        im = Image.open("Tests/images/sugarshack.mpo")
        self.assertEqual(im.n_frames, 2)
        self.assertTrue(im.is_animated)

    def test_eoferror(self):
        im = Image.open("Tests/images/sugarshack.mpo")
        n_frames = im.n_frames

        # Test seeking past the last frame
        self.assertRaises(EOFError, im.seek, n_frames)
        self.assertLess(im.tell(), n_frames)

        # Test that seeking to the last frame does not raise an error
        im.seek(n_frames-1)

    def test_image_grab(self):
        for test_file in test_files:
            im = Image.open(test_file)
            self.assertEqual(im.tell(), 0)
            im0 = im.tobytes()
            im.seek(1)
            self.assertEqual(im.tell(), 1)
            im1 = im.tobytes()
            im.seek(0)
            self.assertEqual(im.tell(), 0)
            im02 = im.tobytes()
            self.assertEqual(im0, im02)
            self.assertNotEqual(im0, im1)

    def test_save(self):
        # Note that only individual frames can be saved at present
        for test_file in test_files:
            im = Image.open(test_file)
            self.assertEqual(im.tell(), 0)
            jpg0 = self.frame_roundtrip(im)
            self.assert_image_similar(im, jpg0, 30)
            im.seek(1)
            self.assertEqual(im.tell(), 1)
            jpg1 = self.frame_roundtrip(im)
            self.assert_image_similar(im, jpg1, 30)


if __name__ == '__main__':
    unittest.main()



from helper import unittest, PillowTestCase, hopper

from PIL import ImagePalette


class TestImagePutPalette(PillowTestCase):

    def test_putpalette(self):
        def palette(mode):
            im = hopper(mode).copy()
            im.putpalette(list(range(256))*3)
            p = im.getpalette()
            if p:
                return im.mode, p[:10]
            return im.mode
        self.assertRaises(ValueError, palette, "1")
        self.assertEqual(palette("L"), ("P", [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]))
        self.assertEqual(palette("P"), ("P", [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]))
        self.assertRaises(ValueError, palette, "I")
        self.assertRaises(ValueError, palette, "F")
        self.assertRaises(ValueError, palette, "RGB")
        self.assertRaises(ValueError, palette, "RGBA")
        self.assertRaises(ValueError, palette, "YCbCr")

    def test_imagepalette(self):
        im = hopper("P")
        im.putpalette(ImagePalette.negative())
        im.putpalette(ImagePalette.random())
        im.putpalette(ImagePalette.sepia())
        im.putpalette(ImagePalette.wedge())


if __name__ == '__main__':
    unittest.main()



from helper import unittest, PillowTestCase, hopper, fromstring, tostring

from io import BytesIO

from PIL import Image
from PIL import ImageFile
from PIL import EpsImagePlugin


codecs = dir(Image.core)

# save original block sizes
MAXBLOCK = ImageFile.MAXBLOCK
SAFEBLOCK = ImageFile.SAFEBLOCK


class TestImageFile(PillowTestCase):

    def test_parser(self):

        def roundtrip(format):

            im = hopper("L").resize((1000, 1000))
            if format in ("MSP", "XBM"):
                im = im.convert("1")

            test_file = BytesIO()

            im.copy().save(test_file, format)

            data = test_file.getvalue()

            parser = ImageFile.Parser()
            parser.feed(data)
            imOut = parser.close()

            return im, imOut

        self.assert_image_equal(*roundtrip("BMP"))
        im1, im2 = roundtrip("GIF")
        self.assert_image_similar(im1.convert('P'), im2, 1)
        self.assert_image_equal(*roundtrip("IM"))
        self.assert_image_equal(*roundtrip("MSP"))
        if "zip_encoder" in codecs:
            try:
                # force multiple blocks in PNG driver
                ImageFile.MAXBLOCK = 8192
                self.assert_image_equal(*roundtrip("PNG"))
            finally:
                ImageFile.MAXBLOCK = MAXBLOCK
        self.assert_image_equal(*roundtrip("PPM"))
        self.assert_image_equal(*roundtrip("TIFF"))
        self.assert_image_equal(*roundtrip("XBM"))
        self.assert_image_equal(*roundtrip("TGA"))
        self.assert_image_equal(*roundtrip("PCX"))

        if EpsImagePlugin.has_ghostscript():
            im1, im2 = roundtrip("EPS")
            # This test fails on Ubuntu 12.04, PPC (Bigendian) It
            # appears to be a ghostscript 9.05 bug, since the
            # ghostscript rendering is wonky and the file is identical
            # to that written on ubuntu 12.04 x64
            # md5sum: ba974835ff2d6f3f2fd0053a23521d4a

            # EPS comes back in RGB:
            self.assert_image_similar(im1, im2.convert('L'), 20)

        if "jpeg_encoder" in codecs:
            im1, im2 = roundtrip("JPEG")  # lossy compression
            self.assert_image(im1, im2.mode, im2.size)

        self.assertRaises(IOError, roundtrip, "PDF")

    def test_ico(self):
        with open('Tests/images/python.ico', 'rb') as f:
            data = f.read()
        with ImageFile.Parser() as p:
            p.feed(data)
            self.assertEqual((48, 48), p.image.size)

    def test_safeblock(self):
        if "zip_encoder" not in codecs:
            self.skipTest("PNG (zlib) encoder not available")

        im1 = hopper()

        try:
            ImageFile.SAFEBLOCK = 1
            im2 = fromstring(tostring(im1, "PNG"))
        finally:
            ImageFile.SAFEBLOCK = SAFEBLOCK

        self.assert_image_equal(im1, im2)

    def test_raise_ioerror(self):
        self.assertRaises(IOError, ImageFile.raise_ioerror, 1)

    def test_raise_typeerror(self):
        with self.assertRaises(TypeError):
            parser = ImageFile.Parser()
            parser.feed(1)

    def test_truncated_with_errors(self):
        if "zip_encoder" not in codecs:
            self.skipTest("PNG (zlib) encoder not available")

        im = Image.open("Tests/images/truncated_image.png")
        with self.assertRaises(IOError):
            im.load()

    def test_truncated_without_errors(self):
        if "zip_encoder" not in codecs:
            self.skipTest("PNG (zlib) encoder not available")

        im = Image.open("Tests/images/truncated_image.png")

        ImageFile.LOAD_TRUNCATED_IMAGES = True
        try:
            im.load()
        finally:
            ImageFile.LOAD_TRUNCATED_IMAGES = False

    def test_broken_datastream_with_errors(self):
        if "zip_encoder" not in codecs:
            self.skipTest("PNG (zlib) encoder not available")

        im = Image.open("Tests/images/broken_data_stream.png")
        with self.assertRaises(IOError):
            im.load()

    def test_broken_datastream_without_errors(self):
        if "zip_encoder" not in codecs:
            self.skipTest("PNG (zlib) encoder not available")

        im = Image.open("Tests/images/broken_data_stream.png")

        ImageFile.LOAD_TRUNCATED_IMAGES = True
        try:
            im.load()
        finally:
            ImageFile.LOAD_TRUNCATED_IMAGES = False


class MockPyDecoder(ImageFile.PyDecoder):
    def decode(self, buffer):
        # eof
        return (-1, 0)


xoff, yoff, xsize, ysize = 10, 20, 100, 100


class MockImageFile(ImageFile.ImageFile):
    def _open(self):
        self.rawmode = 'RGBA'
        self.mode = 'RGBA'
        self.size = (200, 200)
        self.tile = [("MOCK", (xoff, yoff, xoff+xsize, yoff+ysize), 32, None)]


class TestPyDecoder(PillowTestCase):

    def get_decoder(self):
        decoder = MockPyDecoder(None)

        def closure(mode, *args):
            decoder.__init__(mode, *args)
            return decoder

        Image.register_decoder('MOCK', closure)
        return decoder

    def test_setimage(self):
        buf = BytesIO(b'\x00'*255)

        im = MockImageFile(buf)
        d = self.get_decoder()

        im.load()

        self.assertEqual(d.state.xoff, xoff)
        self.assertEqual(d.state.yoff, yoff)
        self.assertEqual(d.state.xsize, xsize)
        self.assertEqual(d.state.ysize, ysize)

        self.assertRaises(ValueError, d.set_as_raw, b'\x00')

    def test_extents_none(self):
        buf = BytesIO(b'\x00'*255)

        im = MockImageFile(buf)
        im.tile = [("MOCK", None, 32, None)]
        d = self.get_decoder()

        im.load()

        self.assertEqual(d.state.xoff, 0)
        self.assertEqual(d.state.yoff, 0)
        self.assertEqual(d.state.xsize, 200)
        self.assertEqual(d.state.ysize, 200)

    def test_negsize(self):
        buf = BytesIO(b'\x00'*255)

        im = MockImageFile(buf)
        im.tile = [("MOCK", (xoff, yoff, -10, yoff+ysize), 32, None)]
        d = self.get_decoder()

        self.assertRaises(ValueError, im.load)

        im.tile = [("MOCK", (xoff, yoff, xoff+xsize, -10), 32, None)]
        self.assertRaises(ValueError, im.load)

    def test_oversize(self):
        buf = BytesIO(b'\x00'*255)

        im = MockImageFile(buf)
        im.tile = [("MOCK", (xoff, yoff, xoff+xsize + 100, yoff+ysize), 32, None)]
        d = self.get_decoder()

        self.assertRaises(ValueError, im.load)

        im.tile = [("MOCK", (xoff, yoff, xoff+xsize, yoff+ysize + 100), 32, None)]
        self.assertRaises(ValueError, im.load)


if __name__ == '__main__':
    unittest.main()



from helper import unittest, PillowTestCase, hopper

from PIL import Image
import os


class TestImage(PillowTestCase):

    def test_image_modes_success(self):
        for mode in [
            '1', 'P', 'PA',
            'L', 'LA', 'La',
            'F', 'I', 'I;16', 'I;16L', 'I;16B', 'I;16N',
            'RGB', 'RGBX', 'RGBA', 'RGBa',
            'CMYK', 'YCbCr', 'LAB', 'HSV',
        ]:
            Image.new(mode, (1, 1))

    def test_image_modes_fail(self):
        for mode in [
            '', 'bad', 'very very long',
            'BGR;15', 'BGR;16', 'BGR;24', 'BGR;32'
        ]:
            with self.assertRaises(ValueError) as e:
                Image.new(mode, (1, 1))
            self.assertEqual(str(e.exception), 'unrecognized image mode')

    def test_sanity(self):

        im = Image.new("L", (100, 100))
        self.assertEqual(
            repr(im)[:45], "<PIL.Image.Image image mode=L size=100x100 at")
        self.assertEqual(im.mode, "L")
        self.assertEqual(im.size, (100, 100))

        im = Image.new("RGB", (100, 100))
        self.assertEqual(
            repr(im)[:45], "<PIL.Image.Image image mode=RGB size=100x100 ")
        self.assertEqual(im.mode, "RGB")
        self.assertEqual(im.size, (100, 100))

        Image.new("L", (100, 100), None)
        im2 = Image.new("L", (100, 100), 0)
        im3 = Image.new("L", (100, 100), "black")

        self.assertEqual(im2.getcolors(), [(10000, 0)])
        self.assertEqual(im3.getcolors(), [(10000, 0)])

        self.assertRaises(ValueError, Image.new, "X", (100, 100))
        self.assertRaises(ValueError, Image.new, "", (100, 100))
        # self.assertRaises(MemoryError, Image.new, "L", (1000000, 1000000))

    def test_width_height(self):
        im = Image.new("RGB", (1, 2))
        self.assertEqual(im.width, 1)
        self.assertEqual(im.height, 2)

        im.size = (3, 4)
        self.assertEqual(im.width, 3)
        self.assertEqual(im.height, 4)

    def test_invalid_image(self):
        if str is bytes:
            import StringIO
            im = StringIO.StringIO('')
        else:
            import io
            im = io.BytesIO(b'')
        self.assertRaises(IOError, Image.open, im)

    def test_bad_mode(self):
        self.assertRaises(ValueError, Image.open, "filename", "bad mode")

    @unittest.skipUnless(Image.HAS_PATHLIB, "requires pathlib/pathlib2")
    def test_pathlib(self):
        from PIL.Image import Path
        im = Image.open(Path("Tests/images/hopper.jpg"))
        self.assertEqual(im.mode, "RGB")
        self.assertEqual(im.size, (128, 128))

        temp_file = self.tempfile("temp.jpg")
        if os.path.exists(temp_file):
            os.remove(temp_file)
        im.save(Path(temp_file))

    def test_fp_name(self):
        temp_file = self.tempfile("temp.jpg")

        class FP(object):
            def write(a, b):
                pass
        fp = FP()
        fp.name = temp_file

        im = hopper()
        im.save(fp)

    def test_tempfile(self):
        # see #1460, pathlib support breaks tempfile.TemporaryFile on py27
        # Will error out on save on 3.0.0
        import tempfile
        im = hopper()
        with tempfile.TemporaryFile() as fp:
            im.save(fp, 'JPEG')
            fp.seek(0)
            reloaded = Image.open(fp)
            self.assert_image_similar(im, reloaded, 20)

    def test_unknown_extension(self):
        im = hopper()
        temp_file = self.tempfile("temp.unknown")
        self.assertRaises(ValueError, im.save, temp_file)

    def test_internals(self):
        im = Image.new("L", (100, 100))
        im.readonly = 1
        im._copy()
        self.assertFalse(im.readonly)

        im.readonly = 1
        im.paste(0, (0, 0, 100, 100))
        self.assertFalse(im.readonly)

    def test_dump(self):
        im = Image.new("L", (10, 10))
        im._dump(self.tempfile("temp_L.ppm"))

        im = Image.new("RGB", (10, 10))
        im._dump(self.tempfile("temp_RGB.ppm"))

        im = Image.new("HSV", (10, 10))
        self.assertRaises(ValueError, im._dump, self.tempfile("temp_HSV.ppm"))

    def test_comparison_with_other_type(self):
        # Arrange
        item = Image.new('RGB', (25, 25), '#000')
        num = 12

        # Act/Assert
        # Shouldn't cause AttributeError (#774)
        self.assertFalse(item is None)
        self.assertFalse(item == None)
        self.assertFalse(item == num)

    def test_expand_x(self):
        # Arrange
        im = hopper()
        orig_size = im.size
        xmargin = 5

        # Act
        im = im._expand(xmargin)

        # Assert
        self.assertEqual(im.size[0], orig_size[0] + 2*xmargin)
        self.assertEqual(im.size[1], orig_size[1] + 2*xmargin)

    def test_expand_xy(self):
        # Arrange
        im = hopper()
        orig_size = im.size
        xmargin = 5
        ymargin = 3

        # Act
        im = im._expand(xmargin, ymargin)

        # Assert
        self.assertEqual(im.size[0], orig_size[0] + 2*xmargin)
        self.assertEqual(im.size[1], orig_size[1] + 2*ymargin)

    def test_getbands(self):
        # Assert
        self.assertEqual(hopper('RGB').getbands(), ('R', 'G', 'B'))
        self.assertEqual(hopper('YCbCr').getbands(), ('Y', 'Cb', 'Cr'))

    def test_getchannel_wrong_params(self):
        im = hopper()

        self.assertRaises(ValueError, im.getchannel, -1)
        self.assertRaises(ValueError, im.getchannel, 3)
        self.assertRaises(ValueError, im.getchannel, 'Z')
        self.assertRaises(ValueError, im.getchannel, '1')

    def test_getchannel(self):
        im = hopper('YCbCr')
        Y, Cb, Cr = im.split()

        self.assert_image_equal(Y, im.getchannel(0))
        self.assert_image_equal(Y, im.getchannel('Y'))
        self.assert_image_equal(Cb, im.getchannel(1))
        self.assert_image_equal(Cb, im.getchannel('Cb'))
        self.assert_image_equal(Cr, im.getchannel(2))
        self.assert_image_equal(Cr, im.getchannel('Cr'))

    def test_getbbox(self):
        # Arrange
        im = hopper()

        # Act
        bbox = im.getbbox()

        # Assert
        self.assertEqual(bbox, (0, 0, 128, 128))

    def test_ne(self):
        # Arrange
        im1 = Image.new('RGB', (25, 25), 'black')
        im2 = Image.new('RGB', (25, 25), 'white')

        # Act / Assert
        self.assertNotEqual(im1, im2)

    def test_alpha_composite(self):
        # https://stackoverflow.com/questions/3374878
        # Arrange
        from PIL import ImageDraw

        expected_colors = sorted([
            (1122, (128, 127, 0, 255)),
            (1089, (0, 255, 0, 255)),
            (3300, (255, 0, 0, 255)),
            (1156, (170, 85, 0, 192)),
            (1122, (0, 255, 0, 128)),
            (1122, (255, 0, 0, 128)),
            (1089, (0, 255, 0, 0))])

        dst = Image.new('RGBA', size=(100, 100), color=(0, 255, 0, 255))
        draw = ImageDraw.Draw(dst)
        draw.rectangle((0, 33, 100, 66), fill=(0, 255, 0, 128))
        draw.rectangle((0, 67, 100, 100), fill=(0, 255, 0, 0))
        src = Image.new('RGBA', size=(100, 100), color=(255, 0, 0, 255))
        draw = ImageDraw.Draw(src)
        draw.rectangle((33, 0, 66, 100), fill=(255, 0, 0, 128))
        draw.rectangle((67, 0, 100, 100), fill=(255, 0, 0, 0))

        # Act
        img = Image.alpha_composite(dst, src)

        # Assert
        img_colors = sorted(img.getcolors())
        self.assertEqual(img_colors, expected_colors)

    def test_alpha_inplace(self):
        src = Image.new('RGBA', (128, 128), 'blue')

        over = Image.new('RGBA', (128, 128), 'red')
        mask = hopper('L')
        over.putalpha(mask)

        target = Image.alpha_composite(src, over)

        # basic
        full = src.copy()
        full.alpha_composite(over)
        self.assert_image_equal(full, target)

        # with offset down to right
        offset = src.copy()
        offset.alpha_composite(over, (64, 64))
        self.assert_image_equal(offset.crop((64, 64, 127, 127)),
                                target.crop((0, 0, 63, 63)))
        self.assertEqual(offset.size, (128, 128))

        # offset and crop
        box = src.copy()
        box.alpha_composite(over, (64, 64), (0, 0, 32, 32))
        self.assert_image_equal(box.crop((64, 64, 96, 96)),
                                target.crop((0, 0, 32, 32)))
        self.assert_image_equal(box.crop((96, 96, 128, 128)),
                                src.crop((0, 0, 32, 32)))
        self.assertEqual(box.size, (128, 128))

        # source point
        source = src.copy()
        source.alpha_composite(over, (32, 32), (32, 32, 96, 96))

        self.assert_image_equal(source.crop((32, 32, 96, 96)),
                                target.crop((32, 32, 96, 96)))
        self.assertEqual(source.size, (128, 128))

        # errors
        self.assertRaises(ValueError,
                          source.alpha_composite, over, "invalid source")
        self.assertRaises(ValueError,
                          source.alpha_composite, over, (0, 0),
                          "invalid destination")
        self.assertRaises(ValueError,
                          source.alpha_composite, over, (0))
        self.assertRaises(ValueError,
                          source.alpha_composite, over, (0, 0), (0))
        self.assertRaises(ValueError,
                          source.alpha_composite, over, (0, -1))
        self.assertRaises(ValueError,
                          source.alpha_composite, over, (0, 0), (0, -1))

    def test_registered_extensions_uninitialized(self):
        # Arrange
        Image._initialized = 0
        extension = Image.EXTENSION
        Image.EXTENSION = {}

        # Act
        Image.registered_extensions()

        # Assert
        self.assertEqual(Image._initialized, 2)

        # Restore the original state and assert
        Image.EXTENSION = extension
        self.assertTrue(Image.EXTENSION)

    def test_registered_extensions(self):
        # Arrange
        # Open an image to trigger plugin registration
        Image.open('Tests/images/rgb.jpg')

        # Act
        extensions = Image.registered_extensions()

        # Assert
        self.assertTrue(extensions)
        for ext in ['.cur', '.icns', '.tif', '.tiff']:
            self.assertIn(ext, extensions)

    def test_effect_mandelbrot(self):
        # Arrange
        size = (512, 512)
        extent = (-3, -2.5, 2, 2.5)
        quality = 100

        # Act
        im = Image.effect_mandelbrot(size, extent, quality)

        # Assert
        self.assertEqual(im.size, (512, 512))
        im2 = Image.open('Tests/images/effect_mandelbrot.png')
        self.assert_image_equal(im, im2)

    def test_effect_mandelbrot_bad_arguments(self):
        # Arrange
        size = (512, 512)
        # Get coordinates the wrong way round:
        extent = (+3, +2.5, -2, -2.5)
        # Quality < 2:
        quality = 1

        # Act/Assert
        self.assertRaises(
            ValueError,
            Image.effect_mandelbrot, size, extent, quality)

    def test_effect_noise(self):
        # Arrange
        size = (100, 100)
        sigma = 128

        # Act
        im = Image.effect_noise(size, sigma)

        # Assert
        self.assertEqual(im.size, (100, 100))
        self.assertEqual(im.mode, "L")
        p0 = im.getpixel((0, 0))
        p1 = im.getpixel((0, 1))
        p2 = im.getpixel((0, 2))
        p3 = im.getpixel((0, 3))
        p4 = im.getpixel((0, 4))
        self.assert_not_all_same([p0, p1, p2, p3, p4])

    def test_effect_spread(self):
        # Arrange
        im = hopper()
        distance = 10

        # Act
        im2 = im.effect_spread(distance)

        # Assert
        self.assertEqual(im.size, (128, 128))
        im3 = Image.open('Tests/images/effect_spread.png')
        self.assert_image_similar(im2, im3, 110)

    def test_check_size(self):
        # Checking that the _check_size function throws value errors
        # when we want it to.
        with self.assertRaises(ValueError):
            Image.new('RGB', 0)  # not a tuple
        with self.assertRaises(ValueError):
            Image.new('RGB', (0,))  # Tuple too short
        with self.assertRaises(ValueError):
            Image.new('RGB', (-1, -1))  # w,h < 0

        # this should pass with 0 sized images, #2259
        im = Image.new('L', (0, 0))
        self.assertEqual(im.size, (0, 0))

        im = Image.new('L', (0, 100))
        self.assertEqual(im.size, (0, 100))

        im = Image.new('L', (100, 0))
        self.assertEqual(im.size, (100, 0))

        self.assertTrue(Image.new('RGB', (1, 1)))
        # Should pass lists too
        i = Image.new('RGB', [1, 1])
        self.assertIsInstance(i.size, tuple)

    def test_storage_neg(self):
        # Storage.c accepted negative values for xsize, ysize.  Was
        # test_neg_ppm, but the core function for that has been
        # removed Calling directly into core to test the error in
        # Storage.c, rather than the size check above

        with self.assertRaises(ValueError):
            Image.core.fill('RGB', (2, -2), (0, 0, 0))

    def test_offset_not_implemented(self):
        # Arrange
        im = hopper()

        # Act / Assert
        self.assertRaises(NotImplementedError, im.offset, None)

    def test_fromstring(self):
        self.assertRaises(NotImplementedError, Image.fromstring)

    def test_linear_gradient_wrong_mode(self):
        # Arrange
        wrong_mode = "RGB"

        # Act / Assert
        self.assertRaises(ValueError,
                          Image.linear_gradient, wrong_mode)

    def test_linear_gradient(self):

        # Arrange
        target_file = "Tests/images/linear_gradient.png"
        for mode in ["L", "P"]:

            # Act
            im = Image.linear_gradient(mode)

            # Assert
            self.assertEqual(im.size, (256, 256))
            self.assertEqual(im.mode, mode)
            self.assertEqual(im.getpixel((0, 0)), 0)
            self.assertEqual(im.getpixel((255, 255)), 255)
            target = Image.open(target_file).convert(mode)
            self.assert_image_equal(im, target)

    def test_radial_gradient_wrong_mode(self):
        # Arrange
        wrong_mode = "RGB"

        # Act / Assert
        self.assertRaises(ValueError,
                          Image.radial_gradient, wrong_mode)

    def test_radial_gradient(self):

        # Arrange
        target_file = "Tests/images/radial_gradient.png"
        for mode in ["L", "P"]:

            # Act
            im = Image.radial_gradient(mode)

            # Assert
            self.assertEqual(im.size, (256, 256))
            self.assertEqual(im.mode, mode)
            self.assertEqual(im.getpixel((0, 0)), 255)
            self.assertEqual(im.getpixel((128, 128)), 0)
            target = Image.open(target_file).convert(mode)
            self.assert_image_equal(im, target)

    def test_register_extensions(self):
        test_format = "a"
        exts = ["b", "c"]
        for ext in exts:
            Image.register_extension(test_format, ext)
        ext_individual = Image.EXTENSION.copy()
        for ext in exts:
            del Image.EXTENSION[ext]

        Image.register_extensions(test_format, exts)
        ext_multiple = Image.EXTENSION.copy()
        for ext in exts:
            del Image.EXTENSION[ext]

        self.assertEqual(ext_individual, ext_multiple)

    def test_remap_palette(self):
        # Test illegal image mode
        im = hopper()
        self.assertRaises(ValueError, im.remap_palette, None)

    def test__new(self):
        from PIL import ImagePalette

        im = hopper('RGB')
        im_p = hopper('P')

        blank_p = Image.new('P', (10, 10))
        blank_pa = Image.new('PA', (10, 10))
        blank_p.palette = None
        blank_pa.palette = None

        def _make_new(base_image, im, palette_result=None):
            new_im = base_image._new(im)
            self.assertEqual(new_im.mode, im.mode)
            self.assertEqual(new_im.size, im.size)
            self.assertEqual(new_im.info, base_image.info)
            if palette_result is not None:
                self.assertEqual(new_im.palette.tobytes(),
                                 palette_result.tobytes())
            else:
                self.assertEqual(new_im.palette, None)

        _make_new(im, im_p, im_p.palette)
        _make_new(im_p, im, None)
        _make_new(im, blank_p, ImagePalette.ImagePalette())
        _make_new(im, blank_pa, ImagePalette.ImagePalette())

    def test_no_resource_warning_on_save(self):
        # https://github.com/python-pillow/Pillow/issues/835
        # Arrange
        test_file = 'Tests/images/hopper.png'
        temp_file = self.tempfile("temp.jpg")

        # Act/Assert
        with Image.open(test_file) as im:
            self.assert_warning(None, im.save, temp_file)


class MockEncoder(object):
    pass


def mock_encode(*args):
    encoder = MockEncoder()
    encoder.args = args
    return encoder


class TestRegistry(PillowTestCase):

    def test_encode_registry(self):

        Image.register_encoder('MOCK', mock_encode)
        self.assertIn('MOCK', Image.ENCODERS)

        enc = Image._getencoder('RGB', 'MOCK', ('args',), extra=('extra',))

        self.assertIsInstance(enc, MockEncoder)
        self.assertEqual(enc.args, ('RGB', 'args', 'extra'))

    def test_encode_registry_fail(self):
        self.assertRaises(IOError, Image._getencoder, 'RGB',
                                                      'DoesNotExist',
                                                      ('args',),
                                                      extra=('extra',))


if __name__ == '__main__':
    unittest.main()



from helper import unittest, PillowTestCase, hopper


class TestImageToBytes(PillowTestCase):

    def test_sanity(self):
        data = hopper().tobytes()
        self.assertIsInstance(data, bytes)


if __name__ == '__main__':
    unittest.main()



#
# The Python Imaging Library.
# $Id$
#
# im.show() drivers
#
# History:
# 2008-04-06 fl   Created
#
# Copyright (c) Secret Labs AB 2008.
#
# See the README file for information on usage and redistribution.
#

from __future__ import print_function

from PIL import Image
import os
import sys

if sys.version_info.major >= 3:
    from shlex import quote
else:
    from pipes import quote

_viewers = []


def register(viewer, order=1):
    try:
        if issubclass(viewer, Viewer):
            viewer = viewer()
    except TypeError:
        pass  # raised if viewer wasn't a class
    if order > 0:
        _viewers.append(viewer)
    elif order < 0:
        _viewers.insert(0, viewer)


def show(image, title=None, **options):
    r"""
    Display a given image.

    :param image: An image object.
    :param title: Optional title.  Not all viewers can display the title.
    :param \**options: Additional viewer options.
    :returns: True if a suitable viewer was found, false otherwise.
    """
    for viewer in _viewers:
        if viewer.show(image, title=title, **options):
            return 1
    return 0


class Viewer(object):
    """Base class for viewers."""

    # main api

    def show(self, image, **options):

        # save temporary image to disk
        if image.mode[:4] == "I;16":
            # @PIL88 @PIL101
            # "I;16" isn't an 'official' mode, but we still want to
            # provide a simple way to show 16-bit images.
            base = "L"
            # FIXME: auto-contrast if max() > 255?
        else:
            base = Image.getmodebase(image.mode)
        if base != image.mode and image.mode != "1" and image.mode != "RGBA":
            image = image.convert(base)

        return self.show_image(image, **options)

    # hook methods

    format = None
    options = {}

    def get_format(self, image):
        """Return format name, or None to save as PGM/PPM"""
        return self.format

    def get_command(self, file, **options):
        raise NotImplementedError

    def save_image(self, image):
        """Save to temporary file, and return filename"""
        return image._dump(format=self.get_format(image), **self.options)

    def show_image(self, image, **options):
        """Display given image"""
        return self.show_file(self.save_image(image), **options)

    def show_file(self, file, **options):
        """Display given file"""
        os.system(self.get_command(file, **options))
        return 1

# --------------------------------------------------------------------


if sys.platform == "win32":

    class WindowsViewer(Viewer):
        format = "BMP"

        def get_command(self, file, **options):
            return ('start "Pillow" /WAIT "%s" '
                    '&& ping -n 2 127.0.0.1 >NUL '
                    '&& del /f "%s"' % (file, file))

    register(WindowsViewer)

elif sys.platform == "darwin":

    class MacViewer(Viewer):
        format = "PNG"
        options = {'compress_level': 1}

        def get_command(self, file, **options):
            # on darwin open returns immediately resulting in the temp
            # file removal while app is opening
            command = "open -a /Applications/Preview.app"
            command = "(%s %s; sleep 20; rm -f %s)&" % (command, quote(file),
                                                        quote(file))
            return command

    register(MacViewer)

else:

    # unixoids

    def which(executable):
        path = os.environ.get("PATH")
        if not path:
            return None
        for dirname in path.split(os.pathsep):
            filename = os.path.join(dirname, executable)
            if os.path.isfile(filename) and os.access(filename, os.X_OK):
                return filename
        return None

    class UnixViewer(Viewer):
        format = "PNG"
        options = {'compress_level': 1}

        def show_file(self, file, **options):
            command, executable = self.get_command_ex(file, **options)
            command = "(%s %s; rm -f %s)&" % (command, quote(file),
                                              quote(file))
            os.system(command)
            return 1

    # implementations

    class DisplayViewer(UnixViewer):
        def get_command_ex(self, file, **options):
            command = executable = "display"
            return command, executable

    if which("display"):
        register(DisplayViewer)

    class EogViewer(UnixViewer):
        def get_command_ex(self, file, **options):
            command = executable = "eog"
            return command, executable

    if which("eog"):
        register(EogViewer)

    class XVViewer(UnixViewer):
        def get_command_ex(self, file, title=None, **options):
            # note: xv is pretty outdated.  most modern systems have
            # imagemagick's display command instead.
            command = executable = "xv"
            if title:
                command += " -name %s" % quote(title)
            return command, executable

    if which("xv"):
        register(XVViewer)

if __name__ == "__main__":

    if len(sys.argv) < 2:
        print("Syntax: python ImageShow.py imagefile [title]")
        sys.exit()

    print(show(Image.open(sys.argv[1]), *sys.argv[2:]))



from helper import unittest, PillowTestCase, hopper, on_appveyor

try:
    from PIL import PyAccess
except ImportError:
    # Skip in setUp()
    pass

from PIL import Image
import sys
import os


class AccessTest(PillowTestCase):
    # initial value
    _init_cffi_access = Image.USE_CFFI_ACCESS
    _need_cffi_access = False

    @classmethod
    def setUpClass(cls):
        Image.USE_CFFI_ACCESS = cls._need_cffi_access

    @classmethod
    def tearDownClass(cls):
        Image.USE_CFFI_ACCESS = cls._init_cffi_access


class TestImagePutPixel(AccessTest):
    def test_sanity(self):
        im1 = hopper()
        im2 = Image.new(im1.mode, im1.size, 0)

        for y in range(im1.size[1]):
            for x in range(im1.size[0]):
                pos = x, y
                im2.putpixel(pos, im1.getpixel(pos))

        self.assert_image_equal(im1, im2)

        im2 = Image.new(im1.mode, im1.size, 0)
        im2.readonly = 1

        for y in range(im1.size[1]):
            for x in range(im1.size[0]):
                pos = x, y
                im2.putpixel(pos, im1.getpixel(pos))

        self.assertFalse(im2.readonly)
        self.assert_image_equal(im1, im2)

        im2 = Image.new(im1.mode, im1.size, 0)

        pix1 = im1.load()
        pix2 = im2.load()

        for y in range(im1.size[1]):
            for x in range(im1.size[0]):
                pix2[x, y] = pix1[x, y]

        self.assert_image_equal(im1, im2)


class TestImageGetPixel(AccessTest):
    @staticmethod
    def color(mode):
        bands = Image.getmodebands(mode)
        if bands == 1:
            return 1
        else:
            return tuple(range(1, bands + 1))

    def check(self, mode, c=None):
        if not c:
            c = self.color(mode)

        # check putpixel
        im = Image.new(mode, (1, 1), None)
        im.putpixel((0, 0), c)
        self.assertEqual(
            im.getpixel((0, 0)), c,
            "put/getpixel roundtrip failed for mode %s, color %s" % (mode, c))

        # Check 0
        im = Image.new(mode, (0, 0), None)
        with self.assertRaises(IndexError):
            im.putpixel((0, 0), c)
        with self.assertRaises(IndexError):
            im.getpixel((0, 0))

        # check initial color
        im = Image.new(mode, (1, 1), c)
        self.assertEqual(
            im.getpixel((0, 0)), c,
            "initial color failed for mode %s, color %s " % (mode, c))

        # Check 0
        im = Image.new(mode, (0, 0), c)
        with self.assertRaises(IndexError):
            im.getpixel((0, 0))

    def test_basic(self):
        for mode in ("1", "L", "LA", "I", "I;16", "I;16B", "F",
                     "P", "PA", "RGB", "RGBA", "RGBX", "CMYK", "YCbCr"):
            self.check(mode)

    def test_signedness(self):
        # see https://github.com/python-pillow/Pillow/issues/452
        # pixelaccess is using signed int* instead of uint*
        for mode in ("I;16", "I;16B"):
            self.check(mode, 2**15-1)
            self.check(mode, 2**15)
            self.check(mode, 2**15+1)
            self.check(mode, 2**16-1)


class TestCffiPutPixel(TestImagePutPixel):
    _need_cffi_access = True

    def setUp(self):
        try:
            import cffi
            assert cffi  # silence warning
        except ImportError:
            self.skipTest("No cffi")


class TestCffiGetPixel(TestImageGetPixel):
    _need_cffi_access = True

    def setUp(self):
        try:
            import cffi
            assert cffi  # silence warning
        except ImportError:
            self.skipTest("No cffi")


class TestCffi(AccessTest):
    _need_cffi_access = True

    def setUp(self):
        try:
            import cffi
            assert cffi  # silence warning
        except ImportError:
            self.skipTest("No cffi")

    def _test_get_access(self, im):
        """Do we get the same thing as the old pixel access

        Using private interfaces, forcing a capi access and
        a pyaccess for the same image"""
        caccess = im.im.pixel_access(False)
        access = PyAccess.new(im, False)

        w, h = im.size
        for x in range(0, w, 10):
            for y in range(0, h, 10):
                self.assertEqual(access[(x, y)], caccess[(x, y)])

        # Access an out-of-range pixel
        self.assertRaises(ValueError,
                          lambda: access[(access.xsize+1, access.ysize+1)])

    def test_get_vs_c(self):
        rgb = hopper('RGB')
        rgb.load()
        self._test_get_access(rgb)
        self._test_get_access(hopper('RGBA'))
        self._test_get_access(hopper('L'))
        self._test_get_access(hopper('LA'))
        self._test_get_access(hopper('1'))
        self._test_get_access(hopper('P'))
        # self._test_get_access(hopper('PA')) # PA -- how do I make a PA image?
        self._test_get_access(hopper('F'))

        im = Image.new('I;16', (10, 10), 40000)
        self._test_get_access(im)
        im = Image.new('I;16L', (10, 10), 40000)
        self._test_get_access(im)
        im = Image.new('I;16B', (10, 10), 40000)
        self._test_get_access(im)

        im = Image.new('I', (10, 10), 40000)
        self._test_get_access(im)
        # These don't actually appear to be modes that I can actually make,
        # as unpack sets them directly into the I mode.
        # im = Image.new('I;32L', (10, 10), -2**10)
        # self._test_get_access(im)
        # im = Image.new('I;32B', (10, 10), 2**10)
        # self._test_get_access(im)

    def _test_set_access(self, im, color):
        """Are we writing the correct bits into the image?

        Using private interfaces, forcing a capi access and
        a pyaccess for the same image"""
        caccess = im.im.pixel_access(False)
        access = PyAccess.new(im, False)

        w, h = im.size
        for x in range(0, w, 10):
            for y in range(0, h, 10):
                access[(x, y)] = color
                self.assertEqual(color, caccess[(x, y)])

        # Attempt to set the value on a read-only image
        access = PyAccess.new(im, True)
        with self.assertRaises(ValueError):
            access[(0, 0)] = color

    def test_set_vs_c(self):
        rgb = hopper('RGB')
        rgb.load()
        self._test_set_access(rgb, (255, 128, 0))
        self._test_set_access(hopper('RGBA'), (255, 192, 128, 0))
        self._test_set_access(hopper('L'), 128)
        self._test_set_access(hopper('LA'), (128, 128))
        self._test_set_access(hopper('1'), 255)
        self._test_set_access(hopper('P'), 128)
        # self._test_set_access(i, (128, 128))  #PA  -- undone how to make
        self._test_set_access(hopper('F'), 1024.0)

        im = Image.new('I;16', (10, 10), 40000)
        self._test_set_access(im, 45000)
        im = Image.new('I;16L', (10, 10), 40000)
        self._test_set_access(im, 45000)
        im = Image.new('I;16B', (10, 10), 40000)
        self._test_set_access(im, 45000)

        im = Image.new('I', (10, 10), 40000)
        self._test_set_access(im, 45000)
        # im = Image.new('I;32L', (10, 10), -(2**10))
        # self._test_set_access(im, -(2**13)+1)
        # im = Image.new('I;32B', (10, 10), 2**10)
        # self._test_set_access(im, 2**13-1)

    def test_not_implemented(self):
        self.assertIsNone(PyAccess.new(hopper("BGR;15")))

    # ref https://github.com/python-pillow/Pillow/pull/2009
    def test_reference_counting(self):
        size = 10

        for _ in range(10):
            # Do not save references to the image, only to the access object
            px = Image.new('L', (size, 1), 0).load()
            for i in range(size):
                # pixels can contain garbage if image is released
                self.assertEqual(px[i, 0], 0)


class TestEmbeddable(unittest.TestCase):
    @unittest.skipIf(not sys.platform.startswith('win32') or
                     sys.version_info[:2] == (3, 4) or
                     on_appveyor(),   # failing on appveyor when run from
                                      # subprocess, not from shell
                     "requires Python 2.7 or >=3.5 for Windows")
    def test_embeddable(self):
        import subprocess
        import ctypes
        from distutils import ccompiler, sysconfig

        with open('embed_pil.c', 'w') as fh:
            fh.write("""
#include "Python.h"

int main(int argc, char* argv[])
{
    char *home = "%s";
#if PY_MAJOR_VERSION >= 3
    wchar_t *whome = Py_DecodeLocale(home, NULL);
    Py_SetPythonHome(whome);
#else
    Py_SetPythonHome(home);
#endif

    Py_InitializeEx(0);
    Py_DECREF(PyImport_ImportModule("PIL.Image"));
    Py_Finalize();

    Py_InitializeEx(0);
    Py_DECREF(PyImport_ImportModule("PIL.Image"));
    Py_Finalize();

#if PY_MAJOR_VERSION >= 3
    PyMem_RawFree(whome);
#endif

    return 0;
}
        """ % sys.prefix.replace('\\', '\\\\'))

        compiler = ccompiler.new_compiler()
        compiler.add_include_dir(sysconfig.get_python_inc())

        libdir = sysconfig.get_config_var('LIBDIR') or sysconfig.get_python_inc().replace('include', 'libs')
        print (libdir)
        compiler.add_library_dir(libdir)
        objects = compiler.compile(['embed_pil.c'])
        compiler.link_executable(objects, 'embed_pil')

        env = os.environ.copy()
        env["PATH"] = sys.prefix + ';' + env["PATH"]

        # do not display the Windows Error Reporting dialog
        ctypes.windll.kernel32.SetErrorMode(0x0002)

        process = subprocess.Popen(['embed_pil.exe'], env=env)
        process.communicate()
        self.assertEqual(process.returncode, 0)


if __name__ == '__main__':
    unittest.main()



from helper import unittest, PillowTestCase, hopper

from PIL import Image
from PIL import ImageShow


class TestImageShow(PillowTestCase):

    def test_sanity(self):
        dir(Image)
        dir(ImageShow)

    def test_register(self):
        # Test registering a viewer that is not a class
        ImageShow.register("not a class")

    def test_show(self):
        class TestViewer:
            methodCalled = False

            def show(self, image, title=None, **options):
                self.methodCalled = True
                return True
        viewer = TestViewer()
        ImageShow.register(viewer, -1)

        im = hopper()
        self.assertTrue(ImageShow.show(im))
        self.assertTrue(viewer.methodCalled)

    def test_viewer(self):
        viewer = ImageShow.Viewer()

        self.assertIsNone(viewer.get_format(None))

        self.assertRaises(NotImplementedError, viewer.get_command, None)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
from helper import unittest, PillowTestCase
from PIL import Image, ImageDraw, ImageFont, features


FONT_SIZE = 20
FONT_PATH = "Tests/fonts/DejaVuSans.ttf"


@unittest.skipUnless(features.check('raqm'), "Raqm Library is not installed.")
class TestImagecomplextext(PillowTestCase):

    def test_english(self):
        #smoke test, this should not fail
        ttf = ImageFont.truetype(FONT_PATH, FONT_SIZE)
        im = Image.new(mode='RGB', size=(300, 100))
        draw = ImageDraw.Draw(im)
        draw.text((0, 0), 'TEST', font=ttf, fill=500, direction='ltr')

    def test_complex_text(self):
        ttf = ImageFont.truetype(FONT_PATH, FONT_SIZE)

        im = Image.new(mode='RGB', size=(300, 100))
        draw = ImageDraw.Draw(im)
        draw.text((0, 0), 'اهلا عمان', font=ttf, fill=500)

        target = 'Tests/images/test_text.png'
        target_img = Image.open(target)

        self.assert_image_similar(im, target_img, .5)

    def test_y_offset(self):
        ttf = ImageFont.truetype("Tests/fonts/NotoNastaliqUrdu-Regular.ttf", FONT_SIZE)

        im = Image.new(mode='RGB', size=(300, 100))
        draw = ImageDraw.Draw(im)
        draw.text((0, 0), 'العالم العربي', font=ttf, fill=500)

        target = 'Tests/images/test_y_offset.png'
        target_img = Image.open(target)

        self.assert_image_similar(im, target_img, 1.7)

    def test_complex_unicode_text(self):
        ttf = ImageFont.truetype(FONT_PATH, FONT_SIZE)

        im = Image.new(mode='RGB', size=(300, 100))
        draw = ImageDraw.Draw(im)
        draw.text((0, 0), 'السلام عليكم', font=ttf, fill=500)

        target = 'Tests/images/test_complex_unicode_text.png'
        target_img = Image.open(target)

        self.assert_image_similar(im, target_img, .5)

    def test_text_direction_rtl(self):
        ttf = ImageFont.truetype(FONT_PATH, FONT_SIZE)

        im = Image.new(mode='RGB', size=(300, 100))
        draw = ImageDraw.Draw(im)
        draw.text((0, 0), 'English عربي', font=ttf, fill=500, direction='rtl')

        target = 'Tests/images/test_direction_rtl.png'
        target_img = Image.open(target)

        self.assert_image_similar(im, target_img, .5)

    def test_text_direction_ltr(self):
        ttf = ImageFont.truetype(FONT_PATH, FONT_SIZE)

        im = Image.new(mode='RGB', size=(300, 100))
        draw = ImageDraw.Draw(im)
        draw.text((0, 0),  'سلطنة عمان Oman', font=ttf, fill=500, direction='ltr')

        target = 'Tests/images/test_direction_ltr.png'
        target_img = Image.open(target)

        self.assert_image_similar(im, target_img, .5)

    def test_text_direction_rtl2(self):
        ttf = ImageFont.truetype(FONT_PATH, FONT_SIZE)

        im = Image.new(mode='RGB', size=(300, 100))
        draw = ImageDraw.Draw(im)
        draw.text((0, 0), 'Oman سلطنة عمان', font=ttf, fill=500, direction='rtl')

        target = 'Tests/images/test_direction_ltr.png'
        target_img = Image.open(target)

        self.assert_image_similar(im, target_img, .5)

    def test_ligature_features(self):
        ttf = ImageFont.truetype(FONT_PATH, FONT_SIZE)

        im = Image.new(mode='RGB', size=(300, 100))
        draw = ImageDraw.Draw(im)
        draw.text((0, 0), 'filling', font=ttf, fill=500, features=['-liga'])
        target = 'Tests/images/test_ligature_features.png'
        target_img = Image.open(target)

        self.assert_image_similar(im, target_img, .5)

        liga_size = ttf.getsize('fi', features=['-liga'])
        self.assertEqual(liga_size, (13, 19))

    def test_kerning_features(self):
        ttf = ImageFont.truetype(FONT_PATH, FONT_SIZE)

        im = Image.new(mode='RGB', size=(300, 100))
        draw = ImageDraw.Draw(im)
        draw.text((0, 0), 'TeToAV', font=ttf, fill=500, features=['-kern'])

        target = 'Tests/images/test_kerning_features.png'
        target_img = Image.open(target)

        self.assert_image_similar(im, target_img, .5)

    def test_arabictext_features(self):
        ttf = ImageFont.truetype(FONT_PATH, FONT_SIZE)

        im = Image.new(mode='RGB', size=(300, 100))
        draw = ImageDraw.Draw(im)
        draw.text((0, 0), 'اللغة العربية', font=ttf, fill=500, features=['-fina', '-init', '-medi'])

        target = 'Tests/images/test_arabictext_features.png'
        target_img = Image.open(target)

        self.assert_image_similar(im, target_img, .5)


if __name__ == '__main__':
    unittest.main()

# End of file



import sys

with open(sys.argv[1], 'r') as fd:
    content = '\n'.join(line.strip() for line in fd if line.strip())
if len(sys.argv) == 3:
    content = content.replace('Win32', sys.argv[2]).replace('x64', sys.argv[2])
with open(sys.argv[1], 'w') as fd:
    fd.write(content)



from helper import unittest, PillowTestCase

from PIL import _binary


class TestBinary(PillowTestCase):

    def test_standard(self):
        self.assertEqual(_binary.i8(b'*'), 42)
        self.assertEqual(_binary.o8(42), b'*')

    def test_little_endian(self):
        self.assertEqual(_binary.i16le(b'\xff\xff\x00\x00'), 65535)
        self.assertEqual(_binary.i32le(b'\xff\xff\x00\x00'), 65535)

        self.assertEqual(_binary.o16le(65535), b'\xff\xff')
        self.assertEqual(_binary.o32le(65535), b'\xff\xff\x00\x00')

    def test_big_endian(self):
        self.assertEqual(_binary.i16be(b'\x00\x00\xff\xff'), 0)
        self.assertEqual(_binary.i32be(b'\x00\x00\xff\xff'), 65535)

        self.assertEqual(_binary.o16be(65535), b'\xff\xff')
        self.assertEqual(_binary.o32be(65535), b'\x00\x00\xff\xff')


if __name__ == '__main__':
    unittest.main()



from helper import unittest, PillowTestCase

from PIL import Image, IcnsImagePlugin

import io
import sys

# sample icon file
TEST_FILE = "Tests/images/pillow.icns"

enable_jpeg2k = hasattr(Image.core, 'jp2klib_version')


class TestFileIcns(PillowTestCase):

    def test_sanity(self):
        # Loading this icon by default should result in the largest size
        # (512x512@2x) being loaded
        im = Image.open(TEST_FILE)
        im.load()
        self.assertEqual(im.mode, "RGBA")
        self.assertEqual(im.size, (1024, 1024))
        self.assertEqual(im.format, "ICNS")

    @unittest.skipIf(sys.platform != 'darwin', "requires macOS")
    def test_save(self):
        im = Image.open(TEST_FILE)

        temp_file = self.tempfile("temp.icns")
        im.save(temp_file)

        reread = Image.open(temp_file)

        self.assertEqual(reread.mode, "RGBA")
        self.assertEqual(reread.size, (1024, 1024))
        self.assertEqual(reread.format, "ICNS")

    @unittest.skipIf(sys.platform != 'darwin', "requires macOS")
    def test_save_append_images(self):
        im = Image.open(TEST_FILE)

        temp_file = self.tempfile("temp.icns")
        provided_im = Image.new('RGBA', (32, 32), (255, 0, 0, 0))
        im.save(temp_file, append_images=[provided_im])

        reread = Image.open(temp_file)
        self.assert_image_equal(reread, im)

        reread = Image.open(temp_file)
        reread.size = (16, 16, 2)
        reread.load()
        self.assert_image_equal(reread, provided_im)

    def test_sizes(self):
        # Check that we can load all of the sizes, and that the final pixel
        # dimensions are as expected
        im = Image.open(TEST_FILE)
        for w, h, r in im.info['sizes']:
            wr = w * r
            hr = h * r
            im2 = Image.open(TEST_FILE)
            im2.size = (w, h, r)
            im2.load()
            self.assertEqual(im2.mode, 'RGBA')
            self.assertEqual(im2.size, (wr, hr))

    def test_older_icon(self):
        # This icon was made with Icon Composer rather than iconutil; it still
        # uses PNG rather than JP2, however (since it was made on 10.9).
        im = Image.open('Tests/images/pillow2.icns')
        for w, h, r in im.info['sizes']:
            wr = w * r
            hr = h * r
            im2 = Image.open('Tests/images/pillow2.icns')
            im2.size = (w, h, r)
            im2.load()
            self.assertEqual(im2.mode, 'RGBA')
            self.assertEqual(im2.size, (wr, hr))

    def test_jp2_icon(self):
        # This icon was made by using Uli Kusterer's oldiconutil to replace
        # the PNG images with JPEG 2000 ones.  The advantage of doing this is
        # that OS X 10.5 supports JPEG 2000 but not PNG; some commercial
        # software therefore does just this.

        # (oldiconutil is here: https://github.com/uliwitness/oldiconutil)

        if not enable_jpeg2k:
            return

        im = Image.open('Tests/images/pillow3.icns')
        for w, h, r in im.info['sizes']:
            wr = w * r
            hr = h * r
            im2 = Image.open('Tests/images/pillow3.icns')
            im2.size = (w, h, r)
            im2.load()
            self.assertEqual(im2.mode, 'RGBA')
            self.assertEqual(im2.size, (wr, hr))

    def test_getimage(self):
        with open(TEST_FILE, 'rb') as fp:
            icns_file = IcnsImagePlugin.IcnsFile(fp)

            im = icns_file.getimage()
            self.assertEqual(im.mode, "RGBA")
            self.assertEqual(im.size, (1024, 1024))

            im = icns_file.getimage((512, 512))
            self.assertEqual(im.mode, "RGBA")
            self.assertEqual(im.size, (512, 512))

    def test_not_an_icns_file(self):
        with io.BytesIO(b'invalid\n') as fp:
            self.assertRaises(SyntaxError,
                              IcnsImagePlugin.IcnsFile, fp)


if __name__ == '__main__':
    unittest.main()



from helper import unittest, PillowTestCase, hopper

from PIL import ImageQt


if ImageQt.qt_is_installed:
    from PIL.ImageQt import qRgba

    def skip_if_qt_is_not_installed(_):
        pass
else:
    def skip_if_qt_is_not_installed(test_case):
        test_case.skipTest('Qt bindings are not installed')


class PillowQtTestCase(object):

    def setUp(self):
        skip_if_qt_is_not_installed(self)

    def tearDown(self):
        pass


class PillowQPixmapTestCase(PillowQtTestCase):

    def setUp(self):
        PillowQtTestCase.setUp(self)
        try:
            if ImageQt.qt_version == '5':
                from PyQt5.QtGui import QGuiApplication
            elif ImageQt.qt_version == '4':
                from PyQt4.QtGui import QGuiApplication
            elif ImageQt.qt_version == 'side':
                from PySide.QtGui import QGuiApplication
        except ImportError:
            self.skipTest('QGuiApplication not installed')

        self.app = QGuiApplication([])

    def tearDown(self):
        PillowQtTestCase.tearDown(self)
        self.app.quit()


class TestImageQt(PillowQtTestCase, PillowTestCase):

    def test_rgb(self):
        # from https://doc.qt.io/qt-4.8/qcolor.html
        # typedef QRgb
        # An ARGB quadruplet on the format #AARRGGBB,
        # equivalent to an unsigned int.
        if ImageQt.qt_version == '5':
            from PyQt5.QtGui import qRgb
        elif ImageQt.qt_version == '4':
            from PyQt4.QtGui import qRgb
        elif ImageQt.qt_version == 'side':
            from PySide.QtGui import qRgb

        self.assertEqual(qRgb(0, 0, 0), qRgba(0, 0, 0, 255))

        def checkrgb(r, g, b):
            val = ImageQt.rgb(r, g, b)
            val = val % 2**24  # drop the alpha
            self.assertEqual(val >> 16, r)
            self.assertEqual(((val >> 8) % 2**8), g)
            self.assertEqual(val % 2**8, b)

        checkrgb(0, 0, 0)
        checkrgb(255, 0, 0)
        checkrgb(0, 255, 0)
        checkrgb(0, 0, 255)

    def test_image(self):
        for mode in ('1', 'RGB', 'RGBA', 'L', 'P'):
            ImageQt.ImageQt(hopper(mode))


if __name__ == '__main__':
    unittest.main()



from helper import unittest, PillowTestCase

from PIL import ImagePalette, Image


class TestImagePalette(PillowTestCase):

    def test_sanity(self):

        ImagePalette.ImagePalette("RGB", list(range(256))*3)
        self.assertRaises(ValueError,
            ImagePalette.ImagePalette, "RGB", list(range(256))*2)

    def test_getcolor(self):

        palette = ImagePalette.ImagePalette()

        test_map = {}
        for i in range(256):
            test_map[palette.getcolor((i, i, i))] = i

        self.assertEqual(len(test_map), 256)
        self.assertRaises(ValueError, palette.getcolor, (1, 2, 3))

        # Test unknown color specifier
        self.assertRaises(ValueError, palette.getcolor, "unknown")

    def test_file(self):

        palette = ImagePalette.ImagePalette("RGB", list(range(256))*3)

        f = self.tempfile("temp.lut")

        palette.save(f)

        p = ImagePalette.load(f)

        # load returns raw palette information
        self.assertEqual(len(p[0]), 768)
        self.assertEqual(p[1], "RGB")

        p = ImagePalette.raw(p[1], p[0])
        self.assertIsInstance(p, ImagePalette.ImagePalette)
        self.assertEqual(p.palette, palette.tobytes())

    def test_make_linear_lut(self):
        # Arrange
        black = 0
        white = 255

        # Act
        lut = ImagePalette.make_linear_lut(black, white)

        # Assert
        self.assertIsInstance(lut, list)
        self.assertEqual(len(lut), 256)
        # Check values
        for i in range(0, len(lut)):
            self.assertEqual(lut[i], i)

    def test_make_linear_lut_not_yet_implemented(self):
        # Update after FIXME
        # Arrange
        black = 1
        white = 255

        # Act
        self.assertRaises(NotImplementedError,
            ImagePalette.make_linear_lut, black, white)

    def test_make_gamma_lut(self):
        # Arrange
        exp = 5

        # Act
        lut = ImagePalette.make_gamma_lut(exp)

        # Assert
        self.assertIsInstance(lut, list)
        self.assertEqual(len(lut), 256)
        # Check a few values
        self.assertEqual(lut[0], 0)
        self.assertEqual(lut[63], 0)
        self.assertEqual(lut[127], 8)
        self.assertEqual(lut[191], 60)
        self.assertEqual(lut[255], 255)

    def test_rawmode_valueerrors(self):
        # Arrange
        palette = ImagePalette.raw("RGB", list(range(256))*3)

        # Act / Assert
        self.assertRaises(ValueError, palette.tobytes)
        self.assertRaises(ValueError, palette.getcolor, (1, 2, 3))
        f = self.tempfile("temp.lut")
        self.assertRaises(ValueError, palette.save, f)

    def test_getdata(self):
        # Arrange
        data_in = list(range(256))*3
        palette = ImagePalette.ImagePalette("RGB", data_in)

        # Act
        mode, data_out = palette.getdata()

        # Assert
        self.assertEqual(mode, "RGB;L")

    def test_rawmode_getdata(self):
        # Arrange
        data_in = list(range(256))*3
        palette = ImagePalette.raw("RGB", data_in)

        # Act
        rawmode, data_out = palette.getdata()

        # Assert
        self.assertEqual(rawmode, "RGB")
        self.assertEqual(data_in, data_out)

    def test_2bit_palette(self):
        # issue #2258, 2 bit palettes are corrupted.
        outfile = self.tempfile('temp.png')

        rgb = b'\x00' * 2 + b'\x01' * 2 + b'\x02' * 2
        img = Image.frombytes('P', (6, 1), rgb)
        img.putpalette(b'\xFF\x00\x00\x00\xFF\x00\x00\x00\xFF')  # RGB
        img.save(outfile, format='PNG')

        reloaded = Image.open(outfile)

        self.assert_image_equal(img, reloaded)

    def test_invalid_palette(self):
        self.assertRaises(IOError,
            ImagePalette.load, "Tests/images/hopper.jpg")


if __name__ == '__main__':
    unittest.main()



from unzip import unzip
from untar import untar
import os

from config import compilers, compiler_from_env, libs


def _relpath(*args):
    return os.path.join(os.getcwd(), *args)


build_dir = _relpath('build')
inc_dir = _relpath('depends')


def check_sig(filename, signame):
    # UNDONE -- need gpg
    return filename


def mkdirs():
    try:
        os.mkdir(build_dir)
    except OSError:
        pass
    try:
        os.mkdir(inc_dir)
    except OSError:
        pass
    for compiler in compilers.values():
        try:
            os.mkdir(os.path.join(inc_dir, compiler['inc_dir']))
        except OSError:
            pass


def extract(src, dest):
    if '.zip' in src:
        return unzip(src, dest)
    if '.tar.gz' in src or '.tgz' in src:
        return untar(src, dest)


def extract_libs():
    for name, lib in libs.items():
        filename = lib['filename']
        if name == 'openjpeg':
            for compiler in compilers.values():
                if not os.path.exists(os.path.join(
                        build_dir, lib['dir']+compiler['inc_dir'])):
                    extract(filename, build_dir)
                    os.rename(os.path.join(build_dir, lib['dir']),
                              os.path.join(
                                  build_dir, lib['dir']+compiler['inc_dir']))
        else:
            extract(filename, build_dir)


def extract_openjpeg(compiler):
    return r"""
rem build openjpeg
setlocal
@echo on
cd %%BUILD%%
mkdir %%INCLIB%%\openjpeg-2.0
copy /Y /B openjpeg-2.0.0-win32-x86\include\openjpeg-2.0  %%INCLIB%%\openjpeg-2.0
copy /Y /B openjpeg-2.0.0-win32-x86\bin\  %%INCLIB%%
copy /Y /B openjpeg-2.0.0-win32-x86\lib\  %%INCLIB%%
endlocal
""" % compiler


def cp_tk(ver_85, ver_86):
    versions = {'ver_85': ver_85, 'ver_86': ver_86}
    return r"""
mkdir %%INCLIB%%\tcl85\include\X11
copy /Y /B %%BUILD%%\tcl%(ver_85)s\generic\*.h %%INCLIB%%\tcl85\include\
copy /Y /B %%BUILD%%\tk%(ver_85)s\generic\*.h %%INCLIB%%\tcl85\include\
copy /Y /B %%BUILD%%\tk%(ver_85)s\xlib\X11\* %%INCLIB%%\tcl85\include\X11\

mkdir %%INCLIB%%\tcl86\include\X11
copy /Y /B %%BUILD%%\tcl%(ver_86)s\generic\*.h %%INCLIB%%\tcl86\include\
copy /Y /B %%BUILD%%\tk%(ver_86)s\generic\*.h %%INCLIB%%\tcl86\include\
copy /Y /B %%BUILD%%\tk%(ver_86)s\xlib\X11\* %%INCLIB%%\tcl86\include\X11\
""" % versions


def header():
    return r"""setlocal
set MSBUILD=C:\Windows\Microsoft.NET\Framework64\v4.0.30319\MSBuild.exe
set CMAKE="cmake.exe"
set INCLIB=%~dp0\depends
set BUILD=%~dp0\build
""" + "\n".join(r'set %s=%%BUILD%%\%s' % (k.upper(), v['dir'])
                for (k, v) in libs.items() if v['dir'])


def setup_compiler(compiler):
    return r"""setlocal EnableDelayedExpansion
call "%%ProgramFiles%%\Microsoft SDKs\Windows\%(env_version)s\Bin\SetEnv.Cmd" /Release %(env_flags)s
set INCLIB=%%INCLIB%%\%(inc_dir)s
""" % compiler


def end_compiler():
    return """
endlocal
"""


def nmake_openjpeg(compiler):
    atts = {'op_ver': '2.1'}
    atts.update(compiler)
    return r"""
rem build openjpeg
setlocal
@echo on
cd /D %%OPENJPEG%%%(inc_dir)s
%%CMAKE%% -DBUILD_THIRDPARTY:BOOL=OFF -G "NMake Makefiles" .
nmake -f Makefile clean
nmake -f Makefile
copy /Y /B bin\* %%INCLIB%%
mkdir %%INCLIB%%\openjpeg-%(op_ver)s
copy /Y /B src\lib\openjp2\*.h %%INCLIB%%\openjpeg-%(op_ver)s
endlocal
""" % atts


def msbuild_openjpeg(compiler):
    atts = {'op_ver': '2.1'}
    atts.update(compiler)
    return r"""
rem build openjpeg
setlocal
@echo on
cd /D %%OPENJPEG%%%(inc_dir)s

%%CMAKE%% -DBUILD_THIRDPARTY:BOOL=OFF -G "NMake Makefiles" .
nmake -f Makefile clean
nmake -f Makefile
copy /Y /B bin\* %%INCLIB%%
mkdir %%INCLIB%%\openjpeg-%(op_ver)s
copy /Y /B src\lib\openjp2\*.h %%INCLIB%%\openjpeg-%(op_ver)s
endlocal
""" % atts


def nmake_libs(compiler):
    # undone -- pre, makes, headers, libs
    return r"""
rem Build libjpeg
setlocal
cd /D %%JPEG%%
nmake -f makefile.vc setup-vc6
nmake -f makefile.vc clean
nmake -f makefile.vc libjpeg.lib
copy /Y /B *.dll %%INCLIB%%
copy /Y /B *.lib %%INCLIB%%
copy /Y /B j*.h %%INCLIB%%
endlocal

rem Build zlib
setlocal
cd /D %%ZLIB%%
nmake -f win32\Makefile.msc clean
nmake -f win32\Makefile.msc zlib.lib
copy /Y /B *.dll %%INCLIB%%
copy /Y /B *.lib %%INCLIB%%
copy /Y /B zlib.lib %%INCLIB%%\z.lib
copy /Y /B zlib.h %%INCLIB%%
copy /Y /B zconf.h %%INCLIB%%
endlocal

rem Build webp
setlocal
cd /D %%WEBP%%
rd /S /Q %%WEBP%%\output\release-static
nmake -f Makefile.vc CFG=release-static RTLIBCFG=static OBJDIR=output all
copy /Y /B output\release-static\%(webp_platform)s\lib\* %%INCLIB%%
mkdir %%INCLIB%%\webp
copy /Y /B src\webp\*.h %%INCLIB%%\\webp
endlocal

rem Build libtiff
setlocal
rem do after building jpeg and zlib
copy %%~dp0\nmake.opt %%TIFF%%

cd /D %%TIFF%%
nmake -f makefile.vc clean
nmake -f makefile.vc lib
copy /Y /B libtiff\*.dll %%INCLIB%%
copy /Y /B libtiff\*.lib %%INCLIB%%
copy /Y /B libtiff\tiff*.h %%INCLIB%%
endlocal


""" % compiler


def msbuild_freetype(compiler):
    if compiler['env_version'] == 'v7.1':
        return msbuild_freetype_71(compiler)
    return msbuild_freetype_70(compiler)


def msbuild_freetype_71(compiler):
    return r"""
rem Build freetype
setlocal
rd /S /Q %%FREETYPE%%\objs
%%MSBUILD%% %%FREETYPE%%\builds\windows\vc%(vc_version)s\freetype.sln /t:Clean;Build /p:Configuration="Release" /p:Platform=%(platform)s /m
xcopy /Y /E /Q %%FREETYPE%%\include %%INCLIB%%
copy /Y /B %%FREETYPE%%\objs\vc%(vc_version)s\%(platform)s\*.lib %%INCLIB%%\freetype.lib
endlocal
""" % compiler


def msbuild_freetype_70(compiler):
    return r"""
rem Build freetype
setlocal
py -3 %%~dp0\fixproj.py %%FREETYPE%%\builds\windows\vc%(vc_version)s\freetype.sln %(platform)s
py -3 %%~dp0\fixproj.py %%FREETYPE%%\builds\windows\vc%(vc_version)s\freetype.vcproj %(platform)s
rd /S /Q %%FREETYPE%%\objs
%%MSBUILD%% %%FREETYPE%%\builds\windows\vc%(vc_version)s\freetype.sln /t:Clean;Build /p:Configuration="LIB Release";Platform=%(platform)s /m
xcopy /Y /E /Q %%FREETYPE%%\include %%INCLIB%%
xcopy /Y /E /Q %%FREETYPE%%\objs\win32\vc%(vc_version)s %%INCLIB%%
copy /Y /B %%FREETYPE%%\objs\win32\vc%(vc_version)s\*.lib %%INCLIB%%\freetype.lib
endlocal
""" % compiler


def build_lcms2(compiler):
    if compiler['env_version'] == 'v7.1':
        return build_lcms_71(compiler)
    return build_lcms_70(compiler)


def build_lcms_70(compiler):
    """Link error here on x64"""
    if compiler['platform'] == 'x64':
        return ''

    """Build LCMS on VC2008. This version is only 32bit/Win32"""
    return r"""
rem Build lcms2
setlocal
rd /S /Q %%LCMS%%\Lib
rd /S /Q %%LCMS%%\Projects\VC%(vc_version)s\Release
%%MSBUILD%% %%LCMS%%\Projects\VC%(vc_version)s\lcms2.sln  /t:Clean /p:Configuration="Release" /p:Platform=Win32 /m
%%MSBUILD%% %%LCMS%%\Projects\VC%(vc_version)s\lcms2.sln /t:lcms2_static /p:Configuration="Release" /p:Platform=Win32 /m
xcopy /Y /E /Q %%LCMS%%\include %%INCLIB%%
copy /Y /B %%LCMS%%\Projects\VC%(vc_version)s\Release\*.lib %%INCLIB%%
endlocal
""" % compiler


def build_lcms_71(compiler):
    return r"""
rem Build lcms2
setlocal
rd /S /Q %%LCMS%%\Lib
rd /S /Q %%LCMS%%\Projects\VC%(vc_version)s\Release
%%MSBUILD%% %%LCMS%%\Projects\VC%(vc_version)s\lcms2.sln  /t:Clean /p:Configuration="Release" /p:Platform=%(platform)s /m
%%MSBUILD%% %%LCMS%%\Projects\VC%(vc_version)s\lcms2.sln  /t:lcms2_static /p:Configuration="Release" /p:Platform=%(platform)s /m
xcopy /Y /E /Q %%LCMS%%\include %%INCLIB%%
copy /Y /B %%LCMS%%\Lib\MS\*.lib %%INCLIB%%
endlocal
""" % compiler


def add_compiler(compiler):
    script.append(setup_compiler(compiler))
    script.append(nmake_libs(compiler))

    # script.append(extract_openjpeg(compiler))

    script.append(msbuild_freetype(compiler))
    script.append(build_lcms2(compiler))
    # script.append(nmake_openjpeg(compiler))
    script.append(end_compiler())


mkdirs()
extract_libs()
script = [header(),
          cp_tk(libs['tk-8.5']['version'],
          libs['tk-8.6']['version'])]


if 'PYTHON' in os.environ:
    add_compiler(compiler_from_env())
else:
    # for compiler in compilers.values():
        # add_compiler(compiler)
    add_compiler(compilers[(7.0, 32)])
    # add_compiler(compilers[(7.1, 64)])

with open('build_deps.cmd', 'w') as f:
    f.write("\n".join(script))



# brute-force search for access descriptor hash table

from __future__ import print_function

modes = [
    "1",
    "L", "LA", "La",
    "I", "I;16", "I;16L", "I;16B", "I;32L", "I;32B",
    "F",
    "P", "PA",
    "RGB", "RGBA", "RGBa", "RGBX",
    "CMYK",
    "YCbCr",
    "LAB", "HSV",
    ]


def hash(s, i):
    # djb2 hash: multiply by 33 and xor character
    for c in s:
        i = (((i << 5) + i) ^ ord(c)) & 0xffffffff
    return i


def check(size, i0):
    h = [None] * size
    for m in modes:
        i = hash(m, i0)
        i = i % size
        if h[i]:
            return 0
        h[i] = m
    return h


min_start = 0

# 1) find the smallest table size with no collisions
for min_size in range(len(modes), 16384):
    if check(min_size, 0):
        print(len(modes), "modes fit in", min_size, "slots")
        break

# 2) see if we can do better with a different initial value
for i0 in range(65556):
    for size in range(1, min_size):
        if check(size, i0):
            if size < min_size:
                print(len(modes), "modes fit in", size, "slots with start", i0)
                min_size = size
                min_start = i0

print()

# print(check(min_size, min_start))

print("#define ACCESS_TABLE_SIZE", min_size)
print("#define ACCESS_TABLE_HASH", min_start)

# for m in modes:
#     print(m, "=>", hash(m, min_start) % min_size)



from helper import unittest, PillowTestCase
from distutils.version import LooseVersion
try:
    import numpy as np
    from numpy.testing import assert_equal

    from scipy import misc
    import scipy
    HAS_SCIPY = True
except ImportError:
    HAS_SCIPY = False


class Test_scipy_resize(PillowTestCase):
    """ Tests for scipy regression in Pillow 2.6.0

    Tests from https://github.com/scipy/scipy/blob/master/scipy/misc/pilutil.py
    """

    def setUp(self):
        if not HAS_SCIPY:
            self.skipTest("Scipy Required")

    def test_imresize(self):
        im = np.random.random((10, 20))
        for T in np.sctypes['float'] + [float]:
            # 1.1 rounds to below 1.1 for float16, 1.101 works
            im1 = misc.imresize(im, T(1.101))
            self.assertEqual(im1.shape, (11, 22))

    # this test fails prior to scipy 0.14.0b1
    # https://github.com/scipy/scipy/commit/855ff1fff805fb91840cf36b7082d18565fc8352
    @unittest.skipIf(HAS_SCIPY and
					 (LooseVersion(scipy.__version__) < LooseVersion('0.14.0')),
                     "Test fails on scipy < 0.14.0")
    def test_imresize4(self):
        im = np.array([[1, 2],
                       [3, 4]])
        res = np.array([[1.,   1.25,  1.75,  2.],
                        [1.5,  1.75,  2.25,  2.5],
                        [2.5,  2.75,  3.25,  3.5],
                        [3.,   3.25,  3.75,  4.]], dtype=np.float32)
        # Check that resizing by target size, float and int are the same
        im2 = misc.imresize(im, (4, 4), mode='F')  # output size
        im3 = misc.imresize(im, 2., mode='F')  # fraction
        im4 = misc.imresize(im, 200, mode='F')  # percentage
        assert_equal(im2, res)
        assert_equal(im3, res)
        assert_equal(im4, res)


if __name__ == '__main__':
    unittest.main()



from helper import unittest, PillowTestCase, hopper

from PIL import Image, SgiImagePlugin


class TestFileSgi(PillowTestCase):

    def test_rgb(self):
        # Created with ImageMagick then renamed:
        # convert hopper.ppm -compress None sgi:hopper.rgb
        test_file = "Tests/images/hopper.rgb"

        im = Image.open(test_file)
        self.assert_image_equal(im, hopper())

    def test_rgb16(self):
        test_file = "Tests/images/hopper16.rgb"

        im = Image.open(test_file)
        self.assert_image_equal(im, hopper())

    def test_l(self):
        # Created with ImageMagick
        # convert hopper.ppm -monochrome -compress None sgi:hopper.bw
        test_file = "Tests/images/hopper.bw"

        im = Image.open(test_file)
        self.assert_image_similar(im, hopper('L'), 2)

    def test_rgba(self):
        # Created with ImageMagick:
        # convert transparent.png -compress None transparent.sgi
        test_file = "Tests/images/transparent.sgi"

        im = Image.open(test_file)
        target = Image.open('Tests/images/transparent.png')
        self.assert_image_equal(im, target)

    def test_rle(self):
        # Created with ImageMagick:
        # convert hopper.ppm  hopper.sgi
        test_file = "Tests/images/hopper.sgi"

        im = Image.open(test_file)
        target = Image.open('Tests/images/hopper.rgb')
        self.assert_image_equal(im, target)

    def test_rle16(self):
        test_file = "Tests/images/tv16.sgi"

        im = Image.open(test_file)
        target = Image.open('Tests/images/tv.rgb')
        self.assert_image_equal(im, target)

    def test_invalid_file(self):
        invalid_file = "Tests/images/flower.jpg"

        self.assertRaises(ValueError,
                          SgiImagePlugin.SgiImageFile, invalid_file)

    def test_write(self):
        def roundtrip(img):
            out = self.tempfile('temp.sgi')
            img.save(out, format='sgi')
            reloaded = Image.open(out)
            self.assert_image_equal(img, reloaded)

        for mode in ('L', 'RGB', 'RGBA'):
            roundtrip(hopper(mode))

        # Test 1 dimension for an L mode image
        roundtrip(Image.new('L', (10, 1)))

    def test_write16(self):
        test_file = "Tests/images/hopper16.rgb"

        im = Image.open(test_file)
        out = self.tempfile('temp.sgi')
        im.save(out, format='sgi', bpc=2)

        reloaded = Image.open(out)
        self.assert_image_equal(im, reloaded)

    def test_unsupported_mode(self):
        im = hopper('LA')
        out = self.tempfile('temp.sgi')

        self.assertRaises(ValueError, im.save, out, format='sgi')


if __name__ == '__main__':
    unittest.main()



from helper import unittest, PillowTestCase, hopper
from test_imageqt import PillowQtTestCase

from PIL import ImageQt, Image


if ImageQt.qt_is_installed:
    from PIL.ImageQt import QImage

    try:
        from PyQt5 import QtGui
        from PyQt5.QtWidgets import QWidget, QHBoxLayout, QLabel, QApplication
        QT_VERSION = 5
    except (ImportError, RuntimeError):
        try:
            from PyQt4 import QtGui
            from PyQt4.QtGui import QWidget, QHBoxLayout, QLabel, QApplication
            QT_VERSION = 4
        except (ImportError, RuntimeError):
            from PySide import QtGui
            from PySide.QtGui import QWidget, QHBoxLayout, QLabel, QApplication
            QT_VERSION = 4


class TestToQImage(PillowQtTestCase, PillowTestCase):

    def test_sanity(self):
        PillowQtTestCase.setUp(self)
        for mode in ('RGB', 'RGBA', 'L', 'P', '1'):
            src = hopper(mode)
            data = ImageQt.toqimage(src)

            self.assertIsInstance(data, QImage)
            self.assertFalse(data.isNull())

            # reload directly from the qimage
            rt = ImageQt.fromqimage(data)
            if mode in ('L', 'P', '1'):
                self.assert_image_equal(rt, src.convert('RGB'))
            else:
                self.assert_image_equal(rt, src)

            if mode == '1':
                # BW appears to not save correctly on QT4 and QT5
                # kicks out errors on console:
                # libpng warning: Invalid color type/bit depth combination in IHDR
                # libpng error: Invalid IHDR data
                continue

            # Test saving the file
            tempfile = self.tempfile('temp_{}.png'.format(mode))
            data.save(tempfile)

            # Check that it actually worked.
            reloaded = Image.open(tempfile)
            # Gray images appear to come back in palette mode.
            # They're roughly equivalent
            if QT_VERSION == 4 and mode == 'L':
                src = src.convert('P')
            self.assert_image_equal(reloaded, src)

    def test_segfault(self):
        PillowQtTestCase.setUp(self)

        app = QApplication([])
        ex = Example()
        assert(app)  # Silence warning
        assert(ex)   # Silence warning


if ImageQt.qt_is_installed:
    class Example(QWidget):

        def __init__(self):
            super(Example, self).__init__()

            img = hopper().resize((1000, 1000))

            qimage = ImageQt.ImageQt(img)

            pixmap1 = QtGui.QPixmap.fromImage(qimage)

            hbox = QHBoxLayout(self)

            lbl = QLabel(self)
            # Segfault in the problem
            lbl.setPixmap(pixmap1.copy())


if __name__ == '__main__':
    unittest.main()



from helper import unittest, PillowTestCase
from PIL import Image


class TestFileFtex(PillowTestCase):

    def test_load_raw(self):
        im = Image.open('Tests/images/ftex_uncompressed.ftu')
        target = Image.open('Tests/images/ftex_uncompressed.png')

        self.assert_image_equal(im, target)

    def test_load_dxt1(self):
        im = Image.open('Tests/images/ftex_dxt1.ftc')
        target = Image.open('Tests/images/ftex_dxt1.png')
        self.assert_image_similar(im, target.convert('RGBA'), 15)


if __name__ == '__main__':
    unittest.main()



import io
import struct

from helper import unittest, PillowTestCase, hopper

from PIL import Image, TiffImagePlugin, TiffTags
from PIL.TiffImagePlugin import _limit_rational, IFDRational

tag_ids = {info.name: info.value for info in TiffTags.TAGS_V2.values()}


class TestFileTiffMetadata(PillowTestCase):

    def test_rt_metadata(self):
        """ Test writing arbitrary metadata into the tiff image directory
            Use case is ImageJ private tags, one numeric, one arbitrary
            data.  https://github.com/python-pillow/Pillow/issues/291
            """

        img = hopper()

        # Behaviour change: re #1416
        # Pre ifd rewrite, ImageJMetaData was being written as a string(2),
        # Post ifd rewrite, it's defined as arbitrary bytes(7). It should
        # roundtrip with the actual bytes, rather than stripped text
        # of the premerge tests.
        #
        # For text items, we still have to decode('ascii','replace') because
        # the tiff file format can't take 8 bit bytes in that field.

        basetextdata = "This is some arbitrary metadata for a text field"
        bindata = basetextdata.encode('ascii') + b" \xff"
        textdata = basetextdata + " " + chr(255)
        reloaded_textdata = basetextdata + " ?"
        floatdata = 12.345
        doubledata = 67.89
        info = TiffImagePlugin.ImageFileDirectory()

        ImageJMetaData = tag_ids['ImageJMetaData']
        ImageJMetaDataByteCounts = tag_ids['ImageJMetaDataByteCounts']
        ImageDescription = tag_ids['ImageDescription']

        info[ImageJMetaDataByteCounts] = len(bindata)
        info[ImageJMetaData] = bindata
        info[tag_ids['RollAngle']] = floatdata
        info.tagtype[tag_ids['RollAngle']] = 11
        info[tag_ids['YawAngle']] = doubledata
        info.tagtype[tag_ids['YawAngle']] = 12

        info[ImageDescription] = textdata

        f = self.tempfile("temp.tif")

        img.save(f, tiffinfo=info)

        loaded = Image.open(f)

        self.assertEqual(loaded.tag[ImageJMetaDataByteCounts], (len(bindata),))
        self.assertEqual(loaded.tag_v2[ImageJMetaDataByteCounts], (len(bindata),))

        self.assertEqual(loaded.tag[ImageJMetaData], bindata)
        self.assertEqual(loaded.tag_v2[ImageJMetaData], bindata)

        self.assertEqual(loaded.tag[ImageDescription], (reloaded_textdata,))
        self.assertEqual(loaded.tag_v2[ImageDescription], reloaded_textdata)

        loaded_float = loaded.tag[tag_ids['RollAngle']][0]
        self.assertAlmostEqual(loaded_float, floatdata, places=5)
        loaded_double = loaded.tag[tag_ids['YawAngle']][0]
        self.assertAlmostEqual(loaded_double, doubledata)

        # check with 2 element ImageJMetaDataByteCounts, issue #2006

        info[ImageJMetaDataByteCounts] = (8, len(bindata) - 8)
        img.save(f, tiffinfo=info)
        loaded = Image.open(f)

        self.assertEqual(loaded.tag[ImageJMetaDataByteCounts], (8, len(bindata) - 8))
        self.assertEqual(loaded.tag_v2[ImageJMetaDataByteCounts], (8, len(bindata) - 8))

    def test_read_metadata(self):
        img = Image.open('Tests/images/hopper_g4.tif')

        self.assertEqual({'YResolution': IFDRational(4294967295, 113653537),
                          'PlanarConfiguration': 1,
                          'BitsPerSample': (1,),
                          'ImageLength': 128,
                          'Compression': 4,
                          'FillOrder': 1,
                          'RowsPerStrip': 128,
                          'ResolutionUnit': 3,
                          'PhotometricInterpretation': 0,
                          'PageNumber': (0, 1),
                          'XResolution': IFDRational(4294967295, 113653537),
                          'ImageWidth': 128,
                          'Orientation': 1,
                          'StripByteCounts': (1968,),
                          'SamplesPerPixel': 1,
                          'StripOffsets': (8,)
                          }, img.tag_v2.named())

        self.assertEqual({'YResolution': ((4294967295, 113653537),),
                          'PlanarConfiguration': (1,),
                          'BitsPerSample': (1,),
                          'ImageLength': (128,),
                          'Compression': (4,),
                          'FillOrder': (1,),
                          'RowsPerStrip': (128,),
                          'ResolutionUnit': (3,),
                          'PhotometricInterpretation': (0,),
                          'PageNumber': (0, 1),
                          'XResolution': ((4294967295, 113653537),),
                          'ImageWidth': (128,),
                          'Orientation': (1,),
                          'StripByteCounts': (1968,),
                          'SamplesPerPixel': (1,),
                          'StripOffsets': (8,)
                          }, img.tag.named())

    def test_write_metadata(self):
        """ Test metadata writing through the python code """
        img = Image.open('Tests/images/hopper.tif')

        f = self.tempfile('temp.tiff')
        img.save(f, tiffinfo=img.tag)

        loaded = Image.open(f)

        original = img.tag_v2.named()
        reloaded = loaded.tag_v2.named()

        for k, v in original.items():
            if isinstance(v, IFDRational):
                original[k] = IFDRational(*_limit_rational(v, 2**31))
            if isinstance(v, tuple) and isinstance(v[0], IFDRational):
                original[k] = tuple([IFDRational(
                                     *_limit_rational(elt, 2**31)) for elt in v])

        ignored = ['StripByteCounts', 'RowsPerStrip',
                   'PageNumber', 'StripOffsets']

        for tag, value in reloaded.items():
            if tag in ignored:
                continue
            if (isinstance(original[tag], tuple)
               and isinstance(original[tag][0], IFDRational)):
                # Need to compare element by element in the tuple,
                # not comparing tuples of object references
                self.assert_deep_equal(original[tag],
                                       value,
                                       "%s didn't roundtrip, %s, %s" %
                                       (tag, original[tag], value))
            else:
                self.assertEqual(original[tag],
                                 value,
                                 "%s didn't roundtrip, %s, %s" %
                                 (tag, original[tag], value))

        for tag, value in original.items():
            if tag not in ignored:
                self.assertEqual(
                    value, reloaded[tag], "%s didn't roundtrip" % tag)

    def test_no_duplicate_50741_tag(self):
        self.assertEqual(tag_ids['MakerNoteSafety'], 50741)
        self.assertEqual(tag_ids['BestQualityScale'], 50780)

    def test_empty_metadata(self):
        f = io.BytesIO(b'II*\x00\x08\x00\x00\x00')
        head = f.read(8)
        info = TiffImagePlugin.ImageFileDirectory(head)
        try:
            self.assert_warning(UserWarning, info.load, f)
        except struct.error:
            self.fail("Should not be struct errors there.")

    def test_iccprofile(self):
        # https://github.com/python-pillow/Pillow/issues/1462
        im = Image.open('Tests/images/hopper.iccprofile.tif')
        out = self.tempfile('temp.tiff')

        im.save(out)
        reloaded = Image.open(out)
        self.assertNotIsInstance(im.info['icc_profile'], tuple)
        self.assertEqual(im.info['icc_profile'], reloaded.info['icc_profile'])

    def test_iccprofile_binary(self):
        # https://github.com/python-pillow/Pillow/issues/1526
        # We should be able to load this, but probably won't be able to save it.

        im = Image.open('Tests/images/hopper.iccprofile_binary.tif')
        self.assertEqual(im.tag_v2.tagtype[34675], 1)
        self.assertTrue(im.info['icc_profile'])

    def test_iccprofile_save_png(self):
        im = Image.open('Tests/images/hopper.iccprofile.tif')
        outfile = self.tempfile('temp.png')
        im.save(outfile)

    def test_iccprofile_binary_save_png(self):
        im = Image.open('Tests/images/hopper.iccprofile_binary.tif')
        outfile = self.tempfile('temp.png')
        im.save(outfile)

    def test_exif_div_zero(self):
        im = hopper()
        info = TiffImagePlugin.ImageFileDirectory_v2()
        info[41988] = TiffImagePlugin.IFDRational(0, 0)

        out = self.tempfile('temp.tiff')
        im.save(out, tiffinfo=info, compression='raw')

        reloaded = Image.open(out)
        self.assertEqual(0, reloaded.tag_v2[41988].numerator)
        self.assertEqual(0, reloaded.tag_v2[41988].denominator)

    def test_expty_values(self):
        data = io.BytesIO(
            b'II*\x00\x08\x00\x00\x00\x03\x00\x1a\x01\x05\x00\x00\x00\x00\x00'
            b'\x00\x00\x00\x00\x1b\x01\x05\x00\x00\x00\x00\x00\x00\x00\x00\x00'
            b'\x98\x82\x02\x00\x07\x00\x00\x002\x00\x00\x00\x00\x00\x00\x00a '
            b'text\x00\x00')
        head = data.read(8)
        info = TiffImagePlugin.ImageFileDirectory_v2(head)
        info.load(data)
        try:
            info = dict(info)
        except ValueError:
            self.fail("Should not be struct value error there.")
        self.assertIn(33432, info)

    def test_PhotoshopInfo(self):
        im = Image.open('Tests/images/issue_2278.tif')

        self.assertIsInstance(im.tag_v2[34377], bytes)
        out = self.tempfile('temp.tiff')
        im.save(out)
        reloaded = Image.open(out)
        self.assertIsInstance(reloaded.tag_v2[34377], bytes)

    def test_too_many_entries(self):
        ifd = TiffImagePlugin.ImageFileDirectory_v2()

        #    277: ("SamplesPerPixel", SHORT, 1),
        ifd._tagdata[277] = struct.pack('hh', 4, 4)
        ifd.tagtype[277] = TiffTags.SHORT

        try:
            self.assert_warning(UserWarning, lambda: ifd[277])
        except ValueError:
            self.fail("Invalid Metadata count should not cause a Value Error.")


if __name__ == '__main__':
    unittest.main()
